@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -155,7 +156,7 @@ void Sql_data_context::deinit() {
   PSI_THREAD_CALL(delete_current_thread)();
 
   PSI_thread *psi =
-      PSI_THREAD_CALL(new_thread)(KEY_thread_x_worker, nullptr, 0);
+      PSI_THREAD_CALL(new_thread)(KEY_thread_x_worker, 0, nullptr, 0);
   PSI_THREAD_CALL(set_thread_os_id)(psi);
   PSI_THREAD_CALL(set_thread)(psi);
 #endif /* HAVE_PSI_THREAD_INTERFACE */
@@ -623,7 +624,7 @@ ngs::Error_code Sql_data_context::execute_prep_stmt(
 }
 
 ngs::Error_code Sql_data_context::reset() {
-  COM_DATA data;
+  COM_DATA data{};
   Empty_resultset rset;
   return execute_server_command(COM_RESET_CONNECTION, data, &rset);
 }
