@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -74,8 +75,7 @@ mysql_harness::Plugin PLUGIN_API harness_plugin_routertestplugin_logger = {
     "Logger",                                // name
     VERSION_NUMBER(1, 0, 0),
     // requires
-    required.size(),
-    required.data(),
+    required.size(), required.data(),
     // conflicts
     0,
     nullptr,  //
@@ -84,7 +84,7 @@ mysql_harness::Plugin PLUGIN_API harness_plugin_routertestplugin_logger = {
     run,      // start
     nullptr,  // stop
     true,     // declares_readiness
-    logger_sink_supported_options.size(),
-    logger_sink_supported_options.data(),
+    logger_sink_supported_options.size(), logger_sink_supported_options.data(),
+    nullptr,  // expose_configuration
 };
 }
