@@ -1,16 +1,17 @@
 /*
-Copyright (c) 2023, Oracle and/or its affiliates.
+Copyright (c) 2023, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is also distributed with certain software (including
+This program is designed to work with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,6 +29,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include <thread>
 
 #include <gmock/gmock.h>
+#include <gtest/gtest.h>
 
 #include "config_builder.h"
 #include "keyring/keyring_manager.h"
@@ -49,15 +51,6 @@ Path g_origin_path;
 
 class ReadReplicaTest : public RouterComponentClusterSetTest {
  protected:
-  void check_log_contains(const ProcessWrapper *proc,
-                          const std::string &expected_string,
-                          size_t expected_occurences) {
-    const std::string log_content = proc->get_logfile_content();
-    EXPECT_EQ(expected_occurences,
-              count_str_occurences(log_content, expected_string))
-        << log_content;
-  }
-
   std::pair<std::string, std::map<std::string, std::string>>
   metadata_cache_section(const std::string &cluster_type_str = "gr") {
     auto ttl_str = std::to_string(std::chrono::duration<double>(kTTL).count());
@@ -146,6 +139,7 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
 
   struct NodeData {
     std::string gr_node_status{"ONLINE"};
+    std::string gr_member_role;
     std::string uuid;
     uint16_t classic_port{};
     uint16_t x_port{};
@@ -188,7 +182,6 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
   };
 
   JsonValue mock_metadata_as_json(const std::string &gr_id, unsigned gr_pos,
-                                  unsigned primary_id,
                                   const std::vector<NodeData> &cluster_nodes,
                                   const std::string &router_options,
                                   const std::string &node_host = "127.0.0.1") {
@@ -205,10 +198,16 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
         continue;
       }
       JsonValue node(rapidjson::kArrayType);
+      node.PushBack(JsonValue(cluster_node.uuid.c_str(),
+                              cluster_node.uuid.length(), allocator),
+                    allocator);
       node.PushBack(static_cast<int>(cluster_node.classic_port), allocator);
       node.PushBack(JsonValue(cluster_node.gr_node_status.c_str(),
                               cluster_node.gr_node_status.length(), allocator),
                     allocator);
+      node.PushBack(JsonValue(cluster_node.gr_member_role.c_str(),
+                              cluster_node.gr_member_role.length(), allocator),
+                    allocator);
       gr_nodes_json.PushBack(node, allocator);
     }
     json_doc.AddMember("gr_nodes", gr_nodes_json, allocator);
@@ -217,6 +216,9 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
     JsonValue cluster_nodes_json(rapidjson::kArrayType);
     for (auto &cluster_node : cluster_nodes) {
       JsonValue node(rapidjson::kArrayType);
+      node.PushBack(JsonValue(cluster_node.uuid.c_str(),
+                              cluster_node.uuid.length(), allocator),
+                    allocator);
       node.PushBack(static_cast<int>(cluster_node.classic_port), allocator);
       node.PushBack(static_cast<int>(cluster_node.x_port), allocator);
       const std::string attributes = cluster_node.get_attributes_as_json_str();
@@ -228,7 +230,6 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
     }
     json_doc.AddMember("cluster_nodes", cluster_nodes_json, allocator);
 
-    json_doc.AddMember("primary_id", static_cast<int>(primary_id), allocator);
     json_doc.AddMember(
         "gr_node_host",
         JsonValue(node_host.c_str(), node_host.length(), allocator), allocator);
@@ -244,12 +245,12 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
   }
 
   void set_mock_metadata(uint16_t http_port, const std::string &gr_id,
-                         unsigned gr_pos, unsigned primary_id,
+                         unsigned gr_pos,
                          const std::vector<NodeData> &cluster_nodes,
                          const std::string &router_options,
                          const std::string &node_host = "127.0.0.1") {
-    const auto json_doc = mock_metadata_as_json(
-        gr_id, gr_pos, primary_id, cluster_nodes, router_options, node_host);
+    const auto json_doc = mock_metadata_as_json(gr_id, gr_pos, cluster_nodes,
+                                                router_options, node_host);
 
     const auto json_str = json_to_string(json_doc);
 
@@ -285,10 +286,12 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
 
     for (size_t i = 0; i < gr_nodes_number + rr_number; ++i) {
       NodeData node;
+      node.uuid = "uuid-" + std::to_string(i + 1);
       node.instance_type =
           i < gr_nodes_number ? "group-member" : "read-replica";
       node.classic_port = port_pool_.get_next_available();
       node.http_port = port_pool_.get_next_available();
+      node.gr_member_role = i == 0 ? "PRIMARY" : "SECONDARY";
 
       cluster_nodes_.emplace_back(node);
     }
@@ -319,6 +322,7 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
 
     for (size_t i = 0; i < ar_nodes_number + rr_number; ++i) {
       NodeData node;
+      node.uuid = "uuid-" + std::to_string(i + 1);
       node.classic_port = port_pool_.get_next_available();
       node.http_port = port_pool_.get_next_available();
       node.instance_type =
@@ -378,6 +382,7 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
     node.instance_type = "read-replica";
     node.classic_port =
         classic_port ? *classic_port : port_pool_.get_next_available();
+    node.uuid = "uuid-" + std::to_string(node.classic_port);
     node.http_port = port_pool_.get_next_available();
     node.process =
         &launch_mysql_server_mock(no_gr_trace_file, node.classic_port,
@@ -426,9 +431,10 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
     EXPECT_THAT(expected_ports_set, ::testing::ContainerEq(used_ports));
   }
 
-  std::vector<std::uint16_t> get_md_servers_classic_ports() const {
+  std::vector<std::uint16_t> get_md_servers_classic_ports(
+      const ClusterSetTopology &cs_topology = {}) const {
     if (is_target_clusterset_) {
-      return clusterset_data_.get_md_servers_classic_ports();
+      return cs_topology.get_md_servers_classic_ports();
     }
 
     std::vector<std::uint16_t> result;
@@ -477,9 +483,10 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
     return result;
   }
 
-  std::vector<uint16_t> get_all_cs_ro_classic_ports(const size_t cluster_id) {
+  std::vector<uint16_t> get_all_cs_ro_classic_ports(
+      const ClusterSetTopology &cs_topology, const size_t cluster_id) {
     std::vector<std::uint16_t> result;
-    const auto &cluster = clusterset_data_.clusters[cluster_id];
+    const auto &cluster = cs_topology.clusters[cluster_id];
     for (const auto [i, node] : stdx::views::enumerate(cluster.nodes)) {
       if (i > 0 || cluster.role == "SECONDARY" || cluster.invalid) {
         result.push_back(node.classic_port);
@@ -493,18 +500,17 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
     return is_target_clusterset_ ? "clusterset-uuid" : "uuid";
   }
 
-  auto &launch_router(const unsigned quarantine_threshold = 1,
+  auto &launch_router(std::vector<std::uint16_t> md_servers,
+                      const unsigned quarantine_threshold = 1,
                       const std::string &configured_cluster_type = "gr",
                       const bool add_static_route = false,
                       const int expected_errorcode = EXIT_SUCCESS,
                       const std::chrono::milliseconds wait_for_notify_ready =
                           kReadyNotifyTimeout) {
     SCOPED_TRACE("// Prepare the dynamic state file for the Router");
-    const auto cluster_all_nodes_ports = get_md_servers_classic_ports();
     router_state_file = create_state_file(
         temp_test_dir_.name(),
-        create_state_file_content("", get_uuid(), cluster_all_nodes_ports,
-                                  view_id_));
+        create_state_file_content("", get_uuid(), md_servers, view_id_));
 
     router_port_rw = port_pool_.get_next_available();
     router_port_ro = port_pool_.get_next_available();
@@ -557,7 +563,7 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
       unsigned gr_pos{0};
       for (auto &cluster_node : cluster_nodes_) {
         if (cluster_node.instance_type == "read-replica") continue;
-        set_mock_metadata(cluster_node.http_port, "", gr_pos, 0, cluster_nodes_,
+        set_mock_metadata(cluster_node.http_port, "", gr_pos, cluster_nodes_,
                           router_options_);
         gr_pos++;
       }
@@ -567,7 +573,7 @@ class ReadReplicaTest : public RouterComponentClusterSetTest {
       assert(*node_id < cluster_nodes_.size());
       for (unsigned i = 0; i < *node_id; i++)
         if (cluster_nodes_[*node_id].instance_type != "read-replica") gr_pos++;
-      set_mock_metadata(cluster_nodes_[*node_id].http_port, "", gr_pos, 0,
+      set_mock_metadata(cluster_nodes_[*node_id].http_port, "", gr_pos,
                         cluster_nodes_, router_options_);
     }
   }
@@ -611,7 +617,7 @@ TEST_F(ReadReplicaTest, ReadOnlyTargetsChanges) {
   const unsigned initial_replica_nodes_count = 1;
 
   create_gr_cluster(initial_gr_nodes_count, initial_replica_nodes_count, "all");
-  const auto &router = launch_router();
+  const auto &router = launch_router(get_md_servers_classic_ports());
 
   // all
   for (size_t i = 0; i <= gr_nodes_count_ + 1; i++) {
@@ -654,7 +660,7 @@ TEST_F(ReadReplicaTest, ReadOnlyTargetsChanges) {
     make_new_connection_ok(router_port_ro, get_gr_ro_classic_ports());
   }
   check_log_contains(
-      &router,
+      router,
       "Error parsing read_only_targets from options JSON string: "
       "Unknown read_only_targets read from the metadata: ''. "
       "Using default value. ({\"read_only_targets\" : \"\"})",
@@ -669,7 +675,7 @@ TEST_F(ReadReplicaTest, ReadOnlyTargetsChanges) {
     make_new_connection_ok(router_port_ro, get_gr_ro_classic_ports());
   }
   check_log_contains(
-      &router,
+      router,
       "Error parsing read_only_targets from options JSON string: "
       "Unknown read_only_targets read from the metadata: 'foo'. "
       "Using default value. ({\"read_only_targets\" : \"foo\"})",
@@ -679,7 +685,7 @@ TEST_F(ReadReplicaTest, ReadOnlyTargetsChanges) {
   change_read_only_targets("all");
   EXPECT_TRUE(
       wait_for_transaction_count_increase(cluster_nodes_[0].http_port, 3));
-  check_log_contains(&router, "Using read_only_targets='all'", 2);
+  check_log_contains(router, "Using read_only_targets='all'", 2);
 
   // make sure Read Replicas were NOT added to the state file as a metadata
   // servers
@@ -698,7 +704,7 @@ TEST_F(ReadReplicaTest, ReadReplicaModeChangesGRWithOnlyRWNode) {
 
   create_gr_cluster(initial_gr_nodes_count, initial_read_replica_nodes_count,
                     "all");
-  launch_router();
+  launch_router(get_md_servers_classic_ports());
 
   make_new_connection_ok(router_port_rw, cluster_nodes_[0].classic_port);
   verify_new_connection_fails(router_port_ro);
@@ -743,7 +749,7 @@ TEST_F(ReadReplicaTest, ReadReplicaInstanceType) {
   const auto read_replica_node_id = gr_nodes_count;
 
   create_gr_cluster(gr_nodes_count, replica_nodes_count, "read_replicas");
-  const auto &router = launch_router();
+  const auto &router = launch_router(get_md_servers_classic_ports());
 
   // the read_only_targets=read_replicas so the Router should only use RR node
   // for RO connections
@@ -773,7 +779,7 @@ TEST_F(ReadReplicaTest, ReadReplicaInstanceType) {
   // connection should not be possible
   verify_new_connection_fails(router_port_ro);
 
-  check_log_contains(&router,
+  check_log_contains(router,
                      "Error parsing instance_type from attributes JSON string: "
                      "Unknown attributes.instance_type value: ''",
                      1);
@@ -785,7 +791,7 @@ TEST_F(ReadReplicaTest, ReadReplicaInstanceType) {
   // connection should not be possible
   verify_new_connection_fails(router_port_ro);
 
-  check_log_contains(&router,
+  check_log_contains(router,
                      "Error parsing instance_type from attributes JSON string: "
                      "Unknown attributes.instance_type value: 'foo'",
                      1);
@@ -801,7 +807,7 @@ TEST_F(ReadReplicaTest, ReadReplicaAddRemove) {
 
   create_gr_cluster(initial_gr_nodes_count, initial_read_replica_nodes_count,
                     "all");
-  launch_router();
+  launch_router(get_md_servers_classic_ports());
 
   EXPECT_TRUE(
       wait_for_transaction_count_increase(cluster_nodes_[0].http_port, 3));
@@ -895,9 +901,10 @@ TEST_P(ReadReplicaQuarantinedTest, ReadReplicaQuarantined) {
 
   create_gr_cluster(gr_nodes_count, read_replica_nodes_count, "read_replicas");
   const unsigned quarantine_threshold = GetParam();
-  const auto &router = launch_router(quarantine_threshold);
+  const auto &router =
+      launch_router(get_md_servers_classic_ports(), quarantine_threshold);
 
-  // remove first read replica [D]
+  SCOPED_TRACE("// remove first read replica [D]");
   const auto classic_port_E = cluster_nodes_[4].classic_port;
   const auto classic_port_D = remove_node(gr_nodes_count, false);
   EXPECT_TRUE(
@@ -909,12 +916,13 @@ TEST_P(ReadReplicaQuarantinedTest, ReadReplicaQuarantined) {
                            cluster_nodes_[gr_nodes_count].classic_port);
   }
 
-  check_log_contains(&router,
-                     "add destination '127.0.0.1:" +
-                         std::to_string(classic_port_D) + "' to quarantine",
-                     1);
+  ASSERT_NO_FATAL_FAILURE(check_log_contains(
+      router,
+      "add destination '127.0.0.1:" + std::to_string(classic_port_D) +
+          "' to quarantine",
+      1));
 
-  // bring back read replica [D]
+  SCOPED_TRACE("// bring back read replica [D]");
   add_read_replica_node(classic_port_D, false, /*position=*/gr_nodes_count);
 
   EXPECT_TRUE(wait_log_contains(
@@ -923,7 +931,7 @@ TEST_P(ReadReplicaQuarantinedTest, ReadReplicaQuarantined) {
           "' is available, remove it from quarantine",
       10s));
 
-  // check RR [D] is back in the rotation
+  SCOPED_TRACE("// check RR [D] is back in the rotation");
   const auto expected_ports =
       std::vector<uint16_t>{classic_port_D, classic_port_E};
   std::vector<std::pair<uint16_t, std::unique_ptr<MySQLSession>>> ro_cons;
@@ -933,32 +941,38 @@ TEST_P(ReadReplicaQuarantinedTest, ReadReplicaQuarantined) {
   }
   check_all_ports_used(expected_ports, ro_cons);
 
-  // remove second read replica [E] now
+  SCOPED_TRACE("// remove second read replica [E] now");
   remove_node(gr_nodes_count + 1, false);
   EXPECT_TRUE(
       wait_for_transaction_count_increase(cluster_nodes_[0].http_port, 3));
 
   for (size_t i = 0; i < 2 * quarantine_threshold + 1; i++) {
-    make_new_connection_ok(router_port_ro, classic_port_D);
+    ASSERT_NO_FATAL_FAILURE(
+        make_new_connection_ok(router_port_ro, classic_port_D));
   }
 
-  check_log_contains(&router,
-                     "add destination '127.0.0.1:" +
-                         std::to_string(classic_port_E) + "' to quarantine",
-                     1);
+  ASSERT_NO_FATAL_FAILURE(check_log_contains(
+      router,
+      "add destination '127.0.0.1:" + std::to_string(classic_port_E) +
+          "' to quarantine",
+      1));
 
-  // remove the first RR [D] again
+  SCOPED_TRACE("// remove the first RR [D] again");
   remove_node(gr_nodes_count, false);
+
+  SCOPED_TRACE("// triggering quarantine: threshold=" +
+               std::to_string(quarantine_threshold));
   for (size_t i = 0; i < quarantine_threshold; i++) {
-    verify_new_connection_fails(router_port_ro);
+    ASSERT_NO_FATAL_FAILURE(verify_new_connection_fails(router_port_ro));
   }
 
-  check_log_contains(&router,
-                     "add destination '127.0.0.1:" +
-                         std::to_string(classic_port_D) + "' to quarantine",
-                     2);
+  ASSERT_NO_FATAL_FAILURE(check_log_contains(
+      router,
+      "add destination '127.0.0.1:" + std::to_string(classic_port_D) +
+          "' to quarantine",
+      2));
 
-  // bring back second RR [E]
+  SCOPED_TRACE("// bring back second RR [E]");
   add_read_replica_node(classic_port_E, false, /*position=*/gr_nodes_count);
 
   EXPECT_TRUE(wait_log_contains(
@@ -968,7 +982,8 @@ TEST_P(ReadReplicaQuarantinedTest, ReadReplicaQuarantined) {
       10s));
 
   for (size_t i = 0; i < 2 * quarantine_threshold; i++) {
-    make_new_connection_ok(router_port_ro, classic_port_E);
+    ASSERT_NO_FATAL_FAILURE(
+        make_new_connection_ok(router_port_ro, classic_port_E));
   }
 }
 
@@ -989,7 +1004,8 @@ TEST_F(ReadReplicaTest, ReadReplicaQuarantinedReadOnlyTargetsAll) {
   create_gr_cluster(initial_gr_nodes_count, initial_read_replica_nodes_count,
                     "all");
   const unsigned quarantine_threshold = 1;
-  const auto &router = launch_router(quarantine_threshold);
+  const auto &router =
+      launch_router(get_md_servers_classic_ports(), quarantine_threshold);
 
   // remove first read replica [D]
   const auto classic_port_D = remove_node(gr_nodes_count_, false);
@@ -1000,7 +1016,7 @@ TEST_F(ReadReplicaTest, ReadReplicaQuarantinedReadOnlyTargetsAll) {
     make_new_connection_ok(router_port_ro, get_all_ro_classic_ports());
   }
 
-  check_log_contains(&router,
+  check_log_contains(router,
                      "add destination '127.0.0.1:" +
                          std::to_string(classic_port_D) + "' to quarantine",
                      1);
@@ -1029,7 +1045,7 @@ TEST_F(ReadReplicaTest, ReadReplicaQuarantinedReadOnlyTargetsAll) {
     make_new_connection_ok(router_port_ro, get_all_ro_classic_ports());
   }
 
-  check_log_contains(&router,
+  check_log_contains(router,
                      "add destination '127.0.0.1:" +
                          std::to_string(classic_port_E) + "' to quarantine",
                      1);
@@ -1043,7 +1059,7 @@ TEST_F(ReadReplicaTest, ReadReplicaQuarantinedReadOnlyTargetsAll) {
     make_new_connection_ok(router_port_ro, {classic_port_B, classic_port_C});
   }
 
-  check_log_contains(&router,
+  check_log_contains(router,
                      "add destination '127.0.0.1:" +
                          std::to_string(classic_port_D) + "' to quarantine",
                      2);
@@ -1077,7 +1093,7 @@ TEST_F(ReadReplicaTest, ReadReplicaInAsyncReplicaCluster) {
   const unsigned ar_ro_nodes_count = ar_nodes_count - ar_rw_nodes_count;
 
   create_ar_cluster(ar_nodes_count, replica_nodes_count);
-  const auto &router = launch_router(1, "rs");
+  const auto &router = launch_router(get_md_servers_classic_ports(), 1, "rs");
 
   // We sleep to verify that the warning that we check is only logged once
   // despite several metadata_cache refresh cycles
@@ -1092,7 +1108,7 @@ TEST_F(ReadReplicaTest, ReadReplicaInAsyncReplicaCluster) {
   const auto classic_port_E = cluster_nodes_[4].classic_port;
   // Read Replicas should be ignored
   for (const auto port : {classic_port_D, classic_port_E}) {
-    check_log_contains(&router,
+    check_log_contains(router,
                        "Ignoring unsupported instance 127.0.0.1:" +
                            std::to_string(port) + ", type: 'read-replica'",
                        1);
@@ -1113,7 +1129,8 @@ TEST_F(ReadReplicaTest, ReadReplicaGRPlusStaticRouting) {
 
   create_gr_cluster(gr_rw_nodes_count, read_replica_nodes_count, "all");
   launch_static_destinations(static_dest_count);
-  launch_router(/*quarantine_threshold*/ 1, "gr",
+  launch_router(get_md_servers_classic_ports(), /*quarantine_threshold*/ 1,
+                "gr",
                 /*add_static_route*/ true);
 
   EXPECT_TRUE(
@@ -1148,7 +1165,8 @@ TEST_F(ReadReplicaTest, ReadReplicaReplicaSetPlusStaticRouting) {
 
   create_ar_cluster(ar_rw_nodes_count, read_replica_nodes_count);
   launch_static_destinations(static_dest_count);
-  launch_router(/*quarantine_threshold*/ 1, "rs",
+  launch_router(get_md_servers_classic_ports(), /*quarantine_threshold*/ 1,
+                "rs",
                 /*add_static_route*/ true);
 
   EXPECT_TRUE(
@@ -1177,19 +1195,22 @@ TEST_F(ReadReplicaTest, ReadReplicaClusterSet) {
   std::string router_options =
       get_router_options_as_json_str("primary", std::nullopt, "all");
 
-  const std::vector<size_t> read_replicas_per_cluster{
+  ClusterSetOptions cs_options;
+  cs_options.tracefile = "metadata_clusterset.js";
+  cs_options.router_options = router_options;
+  cs_options.gr_nodes_number = std::vector<size_t>{3, 3, 3};
+  cs_options.read_replicas_number = std::vector<size_t>{
       primary_read_replicas_nodes_count, replica1_read_replicas_nodes_count, 0};
-  create_clusterset(view_id_, /*target_cluster_id*/ 0,
-                    /*primary_cluster_id*/ 0, "metadata_clusterset.js",
-                    router_options, ".*", false, false,
-                    read_replicas_per_cluster);
+  create_clusterset(cs_options);
+
   is_target_clusterset(true);
 
-  launch_router();
+  launch_router(get_md_servers_classic_ports(cs_options.topology));
 
   // read_only_targets is 'all' so both 2 RO nodes of the Primary Cluster
   // and 2 RRs should be used
-  const auto expected_ports = get_all_cs_ro_classic_ports(0);
+  const auto expected_ports =
+      get_all_cs_ro_classic_ports(cs_options.topology, 0);
   std::vector<std::pair<uint16_t, std::unique_ptr<MySQLSession>>> ro_cons;
   for (size_t i = 0;
        i < 2 * (primary_gr_ro_nodes_count + primary_read_replicas_nodes_count);
@@ -1200,18 +1221,20 @@ TEST_F(ReadReplicaTest, ReadReplicaClusterSet) {
   check_all_ports_used(expected_ports, ro_cons);
 
   // change the target cluster to secondary cluster
-  router_options = get_router_options_as_json_str(
+  cs_options.view_id = ++view_id_;
+  cs_options.target_cluster_id = 1;
+  cs_options.router_options = get_router_options_as_json_str(
       "00000000-0000-0000-0000-0000000000g2", std::nullopt, "all");
-  RouterComponentClusterSetTest::set_mock_metadata_on_all_cs_nodes(
-      ++view_id_,
-      /*target_cluster_id*/ 1, clusterset_data_, router_options);
+
+  set_mock_metadata_on_all_cs_nodes(cs_options);
   EXPECT_TRUE(wait_for_transaction_count_increase(
-      clusterset_data_.clusters[0].nodes[0].http_port, 2));
+      cs_options.topology.clusters[0].nodes[0].http_port, 2));
 
   ro_cons.clear();
   // read_only_targets is 'all' so all 3 RO nodes of the second Cluster and
   // 1 RR should be used
-  const auto expected_ports_2 = get_all_cs_ro_classic_ports(1);
+  const auto expected_ports_2 =
+      get_all_cs_ro_classic_ports(cs_options.topology, 1);
   for (size_t i = 0;
        i < 2 * (replica1_gr_nodes_count + replica1_read_replicas_nodes_count);
        i++) {
@@ -1221,14 +1244,14 @@ TEST_F(ReadReplicaTest, ReadReplicaClusterSet) {
   check_all_ports_used(expected_ports_2, ro_cons);
 
   // change the target cluster back to primary cluster
-  router_options =
+  cs_options.view_id = ++view_id_;
+  cs_options.target_cluster_id = 0;
+  cs_options.router_options =
       get_router_options_as_json_str("primary", std::nullopt, "all");
-  RouterComponentClusterSetTest::set_mock_metadata_on_all_cs_nodes(
-      ++view_id_,
-      /*target_cluster_id*/ 0, clusterset_data_, router_options,
-      router_options);
+  set_mock_metadata_on_all_cs_nodes(cs_options);
+
   EXPECT_TRUE(wait_for_transaction_count_increase(
-      clusterset_data_.clusters[0].nodes[0].http_port, 2));
+      cs_options.topology.clusters[0].nodes[0].http_port, 2));
 
   ro_cons.clear();
   // read_only_targets is 'all' so both 2 RO nodes of the Primary Cluster
@@ -1265,20 +1288,23 @@ TEST_P(ReadReplicaInvalidatedClusterTest,
   const std::string router_options = get_router_options_as_json_str(
       "primary", GetParam().invalidated_cluster_policy, "all");
 
-  const std::vector<size_t> read_replicas_per_cluster{
+  ClusterSetOptions cs_options;
+  cs_options.view_id = view_id_;
+  cs_options.tracefile = "metadata_clusterset.js";
+  cs_options.router_options = router_options;
+  cs_options.gr_nodes_number = std::vector<size_t>{3, 3, 3};
+  cs_options.read_replicas_number = std::vector<size_t>{
       primary_read_replicas_nodes_count, replica1_read_replicas_nodes_count, 0};
-  create_clusterset(view_id_, /*target_cluster_id*/ 0,
-                    /*primary_cluster_id*/ 0, "metadata_clusterset.js",
-                    router_options, ".*", false, false,
-                    read_replicas_per_cluster);
+  create_clusterset(cs_options);
   is_target_clusterset(true);
 
-  launch_router();
+  launch_router(get_md_servers_classic_ports(cs_options.topology));
 
   const auto &rw_con = make_new_connection_ok(
-      router_port_rw, clusterset_data_.clusters[0].nodes[0].classic_port);
+      router_port_rw, cs_options.topology.clusters[0].nodes[0].classic_port);
 
-  const auto expected_ports = get_all_cs_ro_classic_ports(0);
+  const auto expected_ports =
+      get_all_cs_ro_classic_ports(cs_options.topology, 0);
   std::vector<std::pair<uint16_t, std::unique_ptr<MySQLSession>>> ro_cons;
   // read_only_targets is 'all' so both 2 RO nodes of the Primary Cluster
   // and 2 RRs should be used
@@ -1291,12 +1317,12 @@ TEST_P(ReadReplicaInvalidatedClusterTest,
   check_all_ports_used(expected_ports, ro_cons);
 
   // mark the target_cluster as invalid
-  clusterset_data_.clusters[0].invalid = true;
-  RouterComponentClusterSetTest::set_mock_metadata_on_all_cs_nodes(
-      ++view_id_,
-      /*target_cluster_id*/ 0, clusterset_data_, router_options);
+  cs_options.topology.clusters[0].invalid = true;
+  cs_options.view_id = ++view_id_;
+  set_mock_metadata_on_all_cs_nodes(cs_options);
+
   EXPECT_TRUE(wait_for_transaction_count_increase(
-      clusterset_data_.clusters[0].nodes[0].http_port, 2));
+      cs_options.topology.clusters[0].nodes[0].http_port, 2));
 
   verify_existing_connection_dropped(rw_con.get());
   verify_new_connection_fails(router_port_rw);
@@ -1310,7 +1336,8 @@ TEST_P(ReadReplicaInvalidatedClusterTest,
     }
 
     ro_cons.clear();
-    const auto expected_ports_2 = get_all_cs_ro_classic_ports(0);
+    const auto expected_ports_2 =
+        get_all_cs_ro_classic_ports(cs_options.topology, 0);
     for (size_t i = 0; i < 2 * (primary_gr_ro_nodes_count +
                                 primary_read_replicas_nodes_count);
          i++) {
@@ -1352,7 +1379,7 @@ TEST_P(ReadReplicaNoQuorumTest, ReadReplicaGRNoQuorum) {
   const unsigned initial_replica_nodes_count = 2;
 
   create_gr_cluster(initial_gr_nodes_count, initial_replica_nodes_count, "all");
-  /*const auto &router =*/launch_router();
+  /*const auto &router =*/launch_router(get_md_servers_classic_ports());
 
   // all
   for (size_t i = 0; i <= initial_gr_nodes_count + initial_replica_nodes_count;
@@ -1399,7 +1426,7 @@ TEST_F(ReadReplicaTest, HidingNodesReadOnlyTargetsAll) {
   const unsigned gr_nodes_count = 3;
   const unsigned read_replica_nodes_count = 2;
   create_gr_cluster(gr_nodes_count, read_replica_nodes_count, "all");
-  launch_router();
+  launch_router(get_md_servers_classic_ports());
 
   std::vector<std::pair<uint16_t, std::unique_ptr<MySQLSession>>> ro_cons;
 
@@ -1485,7 +1512,7 @@ TEST_F(ReadReplicaTest, HidingNodesReadReplicas) {
   const unsigned replica_nodes_count = 2;
 
   create_gr_cluster(gr_nodes_count, replica_nodes_count, "read_replicas");
-  launch_router();
+  launch_router(get_md_servers_classic_ports());
 
   std::vector<std::pair<uint16_t, std::unique_ptr<MySQLSession>>> ro_cons;
 
@@ -1562,7 +1589,7 @@ TEST_P(MetadataUnavailableTest, MetadataUnavailable) {
   const unsigned replica_nodes_count = 2;
 
   create_gr_cluster(gr_nodes_count, replica_nodes_count, GetParam());
-  auto &router = launch_router();
+  auto &router = launch_router(get_md_servers_classic_ports());
 
   // check that RW/RO ports are open
   make_new_connection_ok(router_port_rw, cluster_nodes_[0].classic_port);
@@ -1589,9 +1616,9 @@ TEST_P(MetadataUnavailableTest, MetadataUnavailable) {
   const std::vector<std::string> expected_log_lines{
       "ERROR .* Failed fetching metadata from any of the 3 metadata servers",
       "INFO .* Stop accepting connections for routing routing:test_default" +
-          rw + " listening on " + rw,
+          rw + " listening on 127.0.0.1:" + rw,
       "INFO .* Stop accepting connections for routing routing:test_default" +
-          ro + " listening on " + ro};
+          ro + " listening on 127.0.0.1:" + ro};
   for (const auto &expected_line : expected_log_lines) {
     EXPECT_TRUE(wait_log_contains(router, expected_line, 5s));
   }
