@@ -61,7 +61,6 @@
 #include "mysqlrouter/classic_protocol_codec_message.h"
 #include "mysqlrouter/classic_protocol_frame.h"
 #include "mysqlrouter/classic_protocol_message.h"
-#include "mysqlrouter/http_request.h"
 #include "mysqlrouter/utils.h"
 #include "openssl_version.h"  // ROUTER_OPENSSL_VERSION
 #include "process_manager.h"
@@ -80,15 +79,7 @@ using namespace std::string_literals;
 using namespace std::chrono_literals;
 using namespace std::string_view_literals;
 
-using ::testing::AllOf;
 using ::testing::AnyOf;
-using ::testing::Contains;
-using ::testing::ElementsAre;
-using ::testing::IsEmpty;
-using ::testing::IsSupersetOf;
-using ::testing::Not;
-using ::testing::Pair;
-using ::testing::SizeIs;
 using ::testing::StartsWith;
 
 static constexpr const auto kIdleServerConnectionsSleepTime{10ms};
@@ -114,24 +105,24 @@ template <size_t N>
 stdx::expected<std::array<std::string, N>, MysqlError> query_one(
     MysqlClient &cli, std::string_view stmt) {
   auto cmd_res = cli.query(stmt);
-  if (!cmd_res) return stdx::make_unexpected(cmd_res.error());
+  if (!cmd_res) return stdx::unexpected(cmd_res.error());
 
   auto results = std::move(*cmd_res);
 
   auto res_it = results.begin();
   if (!(res_it != results.end())) {
-    return stdx::make_unexpected(MysqlError(1, "No results", "HY000"));
+    return stdx::unexpected(MysqlError(1, "No results", "HY000"));
   }
 
   if (res_it->field_count() != N) {
-    return stdx::make_unexpected(
+    return stdx::unexpected(
         MysqlError(1, "field-count doesn't match", "HY000"));
   }
 
   auto rows = res_it->rows();
   auto rows_it = rows.begin();
   if (rows_it == rows.end()) {
-    return stdx::make_unexpected(MysqlError(1, "No rows", "HY000"));
+    return stdx::unexpected(MysqlError(1, "No rows", "HY000"));
   }
 
   std::array<std::string, N> out;
@@ -141,12 +132,12 @@ stdx::expected<std::array<std::string, N>, MysqlError> query_one(
 
   ++rows_it;
   if (rows_it != rows.end()) {
-    return stdx::make_unexpected(MysqlError(1, "Too many rows", "HY000"));
+    return stdx::unexpected(MysqlError(1, "Too many rows", "HY000"));
   }
 
   ++res_it;
   if (res_it != results.end()) {
-    return stdx::make_unexpected(MysqlError(1, "Too many results", "HY000"));
+    return stdx::unexpected(MysqlError(1, "Too many results", "HY000"));
   }
 
   return out;
@@ -158,7 +149,7 @@ static stdx::expected<uint64_t, std::error_code> from_string(
   uint64_t num;
   auto [ptr, ec] = std::from_chars(sv.data(), sv.data() + sv.size(), num);
 
-  if (ec != std::errc{}) return stdx::make_unexpected(make_error_code(ec));
+  if (ec != std::errc{}) return stdx::unexpected(make_error_code(ec));
 
   return num;
 }
@@ -327,7 +318,8 @@ class SharedRouter {
                      {"backend", "file"},
                      {"filename", userfile},
                  })
-        .section("http_server", {{"port", std::to_string(rest_port_)}});
+        .section("http_server", {{"bind_address", "127.0.0.1"},
+                                 {"port", std::to_string(rest_port_)}});
 
     for (const auto &param : share_connection_params) {
       auto port_key =
@@ -353,7 +345,8 @@ class SharedRouter {
               {"client_ssl_key", SSL_TEST_DATA_DIR "/server-key-sha512.pem"},
               {"client_ssl_cert", SSL_TEST_DATA_DIR "/server-cert-sha512.pem"},
               {"connection_sharing", "1"},
-              {"connection_sharing_delay", "0"},
+              {"connection_sharing_delay", "0"},  //
+              {"connect_retry_timeout", "0"},
           });
     }
 
@@ -404,7 +397,7 @@ class SharedRouter {
     // wait for the connections appear in the pool.
     if (param.can_share()) {
       ASSERT_NO_ERROR(wait_for_idle_server_connections(
-          std::min(num_destinations, pool_size_), 1s));
+          std::min(num_destinations, pool_size_), 10s));
     }
   }
 
@@ -416,12 +409,11 @@ class SharedRouter {
 
     if (auto *v = JsonPointer(pointer).Get(json_doc)) {
       if (!v->IsInt()) {
-        return stdx::make_unexpected(
-            make_error_code(std::errc::invalid_argument));
+        return stdx::unexpected(make_error_code(std::errc::invalid_argument));
       }
       return v->GetInt();
     } else {
-      return stdx::make_unexpected(
+      return stdx::unexpected(
           make_error_code(std::errc::no_such_file_or_directory));
     }
   }
@@ -431,6 +423,11 @@ class SharedRouter {
                         "/idleServerConnections");
   }
 
+  stdx::expected<int, std::error_code> stashed_server_connections() {
+    return rest_get_int(rest_api_basepath + "/connection_pool/main/status",
+                        "/stashedServerConnections");
+  }
+
   stdx::expected<void, std::error_code> wait_for_idle_server_connections(
       int expected_value, std::chrono::seconds timeout) {
     using clock_type = std::chrono::steady_clock;
@@ -438,12 +435,31 @@ class SharedRouter {
     const auto end_time = clock_type::now() + timeout;
     do {
       auto int_res = idle_server_connections();
-      if (!int_res) return stdx::make_unexpected(int_res.error());
+      if (!int_res) return stdx::unexpected(int_res.error());
 
       if (*int_res == expected_value) return {};
 
       if (clock_type::now() > end_time) {
-        return stdx::make_unexpected(make_error_code(std::errc::timed_out));
+        return stdx::unexpected(make_error_code(std::errc::timed_out));
+      }
+
+      std::this_thread::sleep_for(kIdleServerConnectionsSleepTime);
+    } while (true);
+  }
+
+  stdx::expected<void, std::error_code> wait_for_stashed_server_connections(
+      int expected_value, std::chrono::seconds timeout) {
+    using clock_type = std::chrono::steady_clock;
+
+    const auto end_time = clock_type::now() + timeout;
+    do {
+      auto int_res = stashed_server_connections();
+      if (!int_res) return stdx::unexpected(int_res.error());
+
+      if (*int_res == expected_value) return {};
+
+      if (clock_type::now() > end_time) {
+        return stdx::unexpected(make_error_code(std::errc::timed_out));
       }
 
       std::this_thread::sleep_for(kIdleServerConnectionsSleepTime);
@@ -557,6 +573,8 @@ class TestEnv : public ::testing::Environment {
         if (s->mysqld_failed_to_start()) {
           GTEST_SKIP() << "mysql-server failed to start.";
         }
+        s->setup_mysqld_accounts();
+        s->install_plugins();
       }
     }
 
@@ -776,7 +794,7 @@ class ShareConnectionTestWithRestartedServer
       }
     }
 
-    ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(0, 1s));
+    ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(0, 10s));
   }
 
  private:
@@ -839,9 +857,17 @@ class ShareConnectionTestTemp
       if (s == nullptr || s->mysqld_failed_to_start()) {
         GTEST_SKIP() << "failed to start mysqld";
       } else {
-        s->flush_privileges();       // reset the auth-cache
-        s->close_all_connections();  // reset the router's connection-pool
-        s->reset_to_defaults();
+        auto admin_cli_res = s->admin_cli();
+        ASSERT_NO_ERROR(admin_cli_res);
+
+        auto cli = std::move(*admin_cli_res);
+
+        // reset the auth-cache
+        SharedServer::flush_privileges(cli);
+
+        // reset the router's connection-pool
+        ASSERT_NO_ERROR(SharedServer::close_all_connections(cli));
+        SharedServer::reset_to_defaults(cli);
       }
     }
   }
@@ -899,8 +925,8 @@ TEST_P(ShareConnectionTestWithRestartedServer,
 
     // wait until connection is in the pool.
     if (can_share) {
-      ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(
-          std::min(ndx + 1, kNumServers), 1s));
+      ASSERT_NO_ERROR(shared_router()->wait_for_stashed_server_connections(
+          std::min(ndx + 1, kNumServers), 10s));
     }
   }
 
@@ -915,20 +941,29 @@ TEST_P(ShareConnectionTestWithRestartedServer,
     ASSERT_NO_FATAL_FAILURE(wait_stopped_intermediate_router(inter));
   }
 
-  // caps for the error-packet parser
-  auto caps = classic_protocol::capabilities::protocol_41;
+  // caps of the server.
+  auto caps = classic_protocol::capabilities::protocol_41 |     // server::Error
+              classic_protocol::capabilities::query_attributes  // client::Query
+      ;
 
   // send one command per connection.
   for (auto [ndx, cli] : stdx::views::enumerate(clis)) {
     SCOPED_TRACE("// testing command " + std::to_string(ndx));
     std::vector<uint8_t> buf;
 
-    {
+    if (ndx == 3) {
+      auto encode_res = classic_protocol::encode<classic_protocol::frame::Frame<
+          classic_protocol::message::client::Query>>({0, {""}}, caps,
+                                                     net::dynamic_buffer(buf));
+      ASSERT_NO_ERROR(encode_res);
+    } else {
       auto encode_res = classic_protocol::encode<
           classic_protocol::frame::Frame<classic_protocol::wire::FixedInt<1>>>(
           {0, {static_cast<uint8_t>(ndx)}}, caps, net::dynamic_buffer(buf));
       ASSERT_NO_ERROR(encode_res);
+    }
 
+    {
       auto send_res = net::impl::socket::send(cli.native_handle(), buf.data(),
                                               buf.size(), 0);
       ASSERT_NO_ERROR(send_res);
@@ -981,8 +1016,6 @@ TEST_P(ShareConnectionTestWithRestartedServer,
 
           auto msg = frame.payload();
 
-          int expected_error_code =
-              2003;  // Can't connect to remove MySQL Server
           switch (ndx) {
             case 0:   // sleep
             case 5:   // create-db
@@ -990,7 +1023,6 @@ TEST_P(ShareConnectionTestWithRestartedServer,
             case 8:   // deprecated
             case 10:  // process-info
             case 11:  // connect
-            case 13:  // debug
             case 15:  // time
             case 16:  // delayed insert
             case cmd_byte<
@@ -1007,7 +1039,7 @@ TEST_P(ShareConnectionTestWithRestartedServer,
             case 39:
 
               // unknown command
-              expected_error_code = 1047;
+              EXPECT_EQ(msg.error_code(), 1047) << msg.message();
               break;
             case cmd_byte<
                 classic_protocol::message::client::StmtExecute>():  // 23
@@ -1016,16 +1048,19 @@ TEST_P(ShareConnectionTestWithRestartedServer,
             case cmd_byte<
                 classic_protocol::message::client::StmtFetch>():  // 28
 
-              // unknown prepared statement handler.
-              expected_error_code = 1243;
+              // unknown prepared statement handler
+              // malformed packet
+              EXPECT_THAT(msg.error_code(), AnyOf(1243, 1835)) << msg.message();
               break;
             case cmd_byte<
                 classic_protocol::message::client::SetOption>():  // 27
-              expected_error_code = 1835;  // malformed packet
+              // malformed packet
+              EXPECT_EQ(msg.error_code(), 1835) << msg.message();
+              break;
+            default:
+              EXPECT_EQ(msg.error_code(), 2003) << msg.message();
               break;
           }
-
-          EXPECT_EQ(msg.error_code(), expected_error_code) << msg.message();
         }
       }
     }
@@ -1070,25 +1105,27 @@ TEST_P(ShareConnectionTestWithRestartedServer,
 
     // wait until connection is in the pool.
     if (can_share) {
-      ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(
-          std::min(ndx + 1, kNumServers), 1s));
+      ASSERT_NO_ERROR(shared_router()->wait_for_stashed_server_connections(
+          std::min(ndx + 1, kNumServers), 10s));
     }
   }
 
-  // caps for the error-packet parser
-  auto caps = classic_protocol::capabilities::protocol_41;
+  // caps of the server.
+  auto caps = classic_protocol::capabilities::protocol_41 |     // server::Error
+              classic_protocol::capabilities::query_attributes  // client::Query
+      ;
 
   // send one command per connection.
   for (auto [ndx, cli] : stdx::views::enumerate(clis)) {
     SCOPED_TRACE("// testing command " + std::to_string(ndx));
     std::vector<uint8_t> buf;
 
-    {
-      auto encode_res = classic_protocol::encode<
-          classic_protocol::frame::Frame<classic_protocol::wire::FixedInt<1>>>(
-          {0, {static_cast<uint8_t>(ndx)}}, caps, net::dynamic_buffer(buf));
-      ASSERT_NO_ERROR(encode_res);
+    auto encode_res = classic_protocol::encode<
+        classic_protocol::frame::Frame<classic_protocol::wire::FixedInt<1>>>(
+        {0, {static_cast<uint8_t>(ndx)}}, caps, net::dynamic_buffer(buf));
+    ASSERT_NO_ERROR(encode_res);
 
+    {
       auto send_res = net::impl::socket::send(cli.native_handle(), buf.data(),
                                               buf.size(), 0);
       ASSERT_NO_ERROR(send_res);
@@ -1140,15 +1177,15 @@ TEST_P(ShareConnectionTestWithRestartedServer,
 
       auto msg = frame.payload();
 
-      int expected_error_code = 1835;  // malformed packet
       switch (ndx) {
         case 0:   // sleep
         case 5:   // create-db
         case 6:   // drop-db
+        case 7:   // refresh
         case 8:   // deprecated
         case 10:  // process-info
         case 11:  // connect
-        case 13:  // debug
+        case 12:  // process-kill
         case 15:  // time
         case 16:  // delayed insert
         case cmd_byte<classic_protocol::message::client::ChangeUser>():  // 17
@@ -1164,42 +1201,54 @@ TEST_P(ShareConnectionTestWithRestartedServer,
         case 39:
 
           // unknown command
-          expected_error_code = 1047;
+          EXPECT_THAT(msg.error_code(), 1047) << msg.message();
           break;
         case cmd_byte<classic_protocol::message::client::StmtExecute>():  // 23
         case cmd_byte<classic_protocol::message::client::StmtReset>():    // 26
         case cmd_byte<classic_protocol::message::client::StmtFetch>():    // 28
-
-          // unknown prepared statement handler | malformed packet.
-          expected_error_code = can_share ? 1243 : 1835;
+          // unknown prepared statement handler
+          // malformed packet
+          EXPECT_THAT(msg.error_code(), AnyOf(1243, 1835)) << msg.message();
           break;
         case cmd_byte<classic_protocol::message::client::InitSchema>():  // 2
 
-          expected_error_code = 1046;  // no database selected
+          // no database selected
+          EXPECT_THAT(msg.error_code(), 1046) << msg.message();
           break;
         case cmd_byte<classic_protocol::message::client::Query>():  // 3
-          expected_error_code =
-              (GetParam().client_ssl_mode != kPassthrough) ? 1065 : 1835;
+
+          // query was empty
+          // malformed packet
+          EXPECT_THAT(msg.error_code(), AnyOf(1065, 1835)) << msg.message();
+          break;
+        case cmd_byte<classic_protocol::message::client::ListFields>():  // 4
+
+          EXPECT_THAT(msg.error_code(), AnyOf(1047,   // unknown command in 9.0
+                                              1835))  // malformed packet in 8.4
+              << msg.message();
           break;
         case cmd_byte<classic_protocol::message::client::StmtPrepare>():  // 22
 
-          expected_error_code = 1065;  // query was empty
+          // query was empty
+          EXPECT_THAT(msg.error_code(), 1065) << msg.message();
           break;
         case cmd_byte<classic_protocol::message::client::BinlogDump>():  // 18
         case cmd_byte<
             classic_protocol::message::client::BinlogDumpGtid>():  // 30
-
-          expected_error_code = 1227;  // query was empty
+        case 13:                                                   // debug
+                  // access denied; SUPER is needed.
+          EXPECT_THAT(msg.error_code(), 1227) << msg.message();
           break;
         case cmd_byte<
             classic_protocol::message::client::RegisterReplica>():  // 21
 
           // access denied
-          expected_error_code = 1045;
+          EXPECT_THAT(msg.error_code(), 1045) << msg.message();
+          break;
+        default:
+          EXPECT_THAT(msg.error_code(), 1835) << msg.message();
           break;
       }
-
-      EXPECT_EQ(msg.error_code(), expected_error_code) << msg.message();
     } else if (expected_response == ExpectedResponse::Ok) {
       buf.resize(1024);  // should be large enough.
 
@@ -1264,7 +1313,7 @@ TEST_P(ShareConnectionTestWithRestartedServer,
 
   if (can_share) {
     SCOPED_TRACE("// wait until connection is pooled.");
-    ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(1, 1s));
+    ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(1, 10s));
 
     SCOPED_TRACE("// force a close of the connections in the pool");
 
@@ -1309,7 +1358,8 @@ TEST_P(ShareConnectionTestWithRestartedServer,
     EXPECT_EQ(my_port, *my_port_num_res);
 
     if (can_share) {
-      ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(1, 1s));
+      ASSERT_NO_ERROR(
+          shared_router()->wait_for_stashed_server_connections(1, 10s));
 
       this->wait_for_connections_to_server_expired(my_port);
     }
@@ -1435,7 +1485,7 @@ TEST_P(ShareConnectionTestWithRestartedServer,
   }
 
   if (can_share) {
-    ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(1, 1s));
+    ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(1, 10s));
   }
 
   {
@@ -1473,7 +1523,7 @@ TEST_P(ShareConnectionTestWithRestartedServer,
   }
 
   if (can_share) {
-    ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(1, 1s));
+    ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(1, 10s));
   }
 
   // stop the first router and start another again.
@@ -1596,7 +1646,8 @@ TEST_P(ShareConnectionTestWithRestartedServer,
     my_port = *my_port_num_res;
 
     if (can_share) {
-      ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(1, 1s));
+      ASSERT_NO_ERROR(
+          shared_router()->wait_for_stashed_server_connections(1, 10s));
 
       ASSERT_NO_FATAL_FAILURE(
           this->wait_for_connections_to_server_expired(my_port));
@@ -1726,7 +1777,8 @@ TEST_P(ShareConnectionTestWithRestartedServer,
     my_port = *my_port_num_res;
 
     if (can_share) {
-      ASSERT_NO_ERROR(shared_router()->wait_for_idle_server_connections(1, 1s));
+      ASSERT_NO_ERROR(
+          shared_router()->wait_for_stashed_server_connections(1, 10s));
     }
 
     // reconnects
@@ -1837,17 +1889,34 @@ TEST_P(ShareConnectionTestWithRestartedServer,
  */
 TEST_P(ShareConnectionTestWithRestartedServer,
        classic_protocol_kill_my_backend_reconnect_select) {
+  const bool can_share = GetParam().can_share();
+
   SCOPED_TRACE("// connecting to server");
-  std::array<MysqlClient, 4> clis;  // more clients then destinations.
+  std::array<MysqlClient, 4> clis;  // one per destination
+
+  for (auto [ndx, cli] : stdx::views::enumerate(clis)) {
+    if (can_share && ndx == 3) {
+      // wait for all connections to be pooled.
+      ASSERT_NO_ERROR(
+          shared_router()->wait_for_stashed_server_connections(3, 10s));
+    }
 
-  for (auto &cli : clis) {
     cli.username("root");
     cli.password("");
 
+    // ndx=3 uses a pooled connection.
     ASSERT_NO_ERROR(cli.connect(shared_router()->host(),
                                 shared_router()->port(GetParam())));
   }
 
+  if (can_share) {
+    // wait for ndx=3 to be back in the pool.
+    ASSERT_NO_ERROR(
+        shared_router()->wait_for_stashed_server_connections(3, 10s));
+  }
+
+  SCOPED_TRACE("// querying port of first server");
+
   auto port_res = query_one<1>(clis[0], "SELECT @@port");
   ASSERT_NO_ERROR(port_res);
 
@@ -1856,14 +1925,20 @@ TEST_P(ShareConnectionTestWithRestartedServer,
 
   uint16_t my_port = *my_port_num_res;
 
-  // shut down the server connection is for while the connection is pooled.
+  if (can_share) {
+    // wait for clis[0] to be back in the pool again.
+    ASSERT_NO_ERROR(
+        shared_router()->wait_for_stashed_server_connections(3, 10s));
+  }
+
+  // shut down the server connection while the connection is pooled.
   // wait for the server to shutdown
   int nodes_shutdown{0};
   // shut down the intermediate router while the connection is pooled
 
   for (auto [ndx, s] : stdx::views::enumerate(shared_servers())) {
     if (s->server_port() == my_port) {
-      auto inter = intermediate_routers()[ndx];
+      auto *inter = intermediate_routers()[ndx];
 
       ASSERT_NO_FATAL_FAILURE(this->stop_intermediate_router(inter));
 
@@ -1876,7 +1951,7 @@ TEST_P(ShareConnectionTestWithRestartedServer,
   {
     auto cmd_res = query_one<1>(clis[0], "SELECT @@port");
     ASSERT_ERROR(cmd_res);
-    if (!GetParam().can_share()) {
+    if (!can_share) {
       // not pooled, the connection is closed directly.
       EXPECT_EQ(cmd_res.error().value(), 2013) << cmd_res.error();
       EXPECT_THAT(cmd_res.error().message(),
@@ -1907,7 +1982,7 @@ TEST_P(ShareConnectionTestWithRestartedServer,
     auto cmd_res = query_one<1>(clis[3], "SELECT @@port");
     ASSERT_ERROR(cmd_res);
 
-    if (!GetParam().can_share()) {
+    if (!can_share) {
       // not pooled, the connection is closed directly.
       EXPECT_EQ(cmd_res.error().value(), 2013) << cmd_res.error();
       EXPECT_THAT(cmd_res.error().message(),
