@@ -23,8 +23,8 @@
 
 #include "sql/rpl_trx_boundary_parser.h"
 
-#include "my_loglevel.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/my_loglevel.h"
 #include "mysqld_error.h"
 #include "sql/log.h"
 
