@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_13732547_1702565708_68_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
-# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_13732547_1702565708_68_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_15014491_1712730762_01_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_15014491_1712730762_01_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
 #ifndef MY_SQL_PARSER_DEBUG
 # if defined YYDEBUG
@@ -345,29 +345,29 @@ extern int my_sql_parser_debug;
     LOOP_SYM = 547,
     LOW_PRIORITY = 548,
     LT = 549,
-    MASTER_AUTO_POSITION_SYM = 550,
-    MASTER_BIND_SYM = 551,
-    MASTER_CONNECT_RETRY_SYM = 552,
-    MASTER_DELAY_SYM = 553,
-    MASTER_HOST_SYM = 554,
-    MASTER_LOG_FILE_SYM = 555,
-    MASTER_LOG_POS_SYM = 556,
-    MASTER_PASSWORD_SYM = 557,
-    MASTER_PORT_SYM = 558,
-    MASTER_RETRY_COUNT_SYM = 559,
-    MASTER_SSL_CAPATH_SYM = 561,
-    MASTER_TLS_VERSION_SYM = 562,
-    MASTER_SSL_CA_SYM = 563,
-    MASTER_SSL_CERT_SYM = 564,
-    MASTER_SSL_CIPHER_SYM = 565,
-    MASTER_SSL_CRL_SYM = 566,
-    MASTER_SSL_CRLPATH_SYM = 567,
-    MASTER_SSL_KEY_SYM = 568,
-    MASTER_SSL_SYM = 569,
-    MASTER_SSL_VERIFY_SERVER_CERT_SYM = 570,
+    OBSOLETE_TOKEN_550 = 550,
+    OBSOLETE_TOKEN_551 = 551,
+    OBSOLETE_TOKEN_552 = 552,
+    OBSOLETE_TOKEN_553 = 553,
+    OBSOLETE_TOKEN_554 = 554,
+    OBSOLETE_TOKEN_555 = 555,
+    OBSOLETE_TOKEN_556 = 556,
+    OBSOLETE_TOKEN_557 = 557,
+    OBSOLETE_TOKEN_558 = 558,
+    OBSOLETE_TOKEN_559 = 559,
+    OBSOLETE_TOKEN_561 = 561,
+    OBSOLETE_TOKEN_562 = 562,
+    OBSOLETE_TOKEN_563 = 563,
+    OBSOLETE_TOKEN_564 = 564,
+    OBSOLETE_TOKEN_565 = 565,
+    OBSOLETE_TOKEN_566 = 566,
+    OBSOLETE_TOKEN_567 = 567,
+    OBSOLETE_TOKEN_568 = 568,
+    OBSOLETE_TOKEN_569 = 569,
+    OBSOLETE_TOKEN_570 = 570,
     MASTER_SYM = 571,
-    MASTER_USER_SYM = 572,
-    MASTER_HEARTBEAT_PERIOD_SYM = 573,
+    OBSOLETE_TOKEN_572 = 572,
+    OBSOLETE_TOKEN_573 = 573,
     MATCH = 574,
     MAX_CONNECTIONS_PER_HOUR = 575,
     MAX_QUERIES_PER_HOUR = 576,
@@ -760,8 +760,8 @@ extern int my_sql_parser_debug;
     RESOURCE_SYM = 963,
     SYSTEM_SYM = 964,
     VCPU_SYM = 965,
-    MASTER_PUBLIC_KEY_PATH_SYM = 966,
-    GET_MASTER_PUBLIC_KEY_SYM = 967,
+    OBSOLETE_TOKEN_966 = 966,
+    OBSOLETE_TOKEN_967 = 967,
     RESTART_SYM = 968,
     DEFINITION_SYM = 969,
     DESCRIPTION_SYM = 970,
@@ -783,10 +783,10 @@ extern int my_sql_parser_debug;
     OJ_SYM = 986,
     NETWORK_NAMESPACE_SYM = 987,
     RANDOM_SYM = 988,
-    MASTER_COMPRESSION_ALGORITHM_SYM = 989,
-    MASTER_ZSTD_COMPRESSION_LEVEL_SYM = 990,
+    OBSOLETE_TOKEN_989 = 989,
+    OBSOLETE_TOKEN_990 = 990,
     PRIVILEGE_CHECKS_USER_SYM = 991,
-    MASTER_TLS_CIPHERSUITES_SYM = 992,
+    OBSOLETE_TOKEN_992 = 992,
     REQUIRE_ROW_FORMAT_SYM = 993,
     PASSWORD_LOCK_TIME_SYM = 994,
     FAILED_LOGIN_ATTEMPTS_SYM = 995,
@@ -855,11 +855,15 @@ extern int my_sql_parser_debug;
     PARALLEL_SYM = 1208,
     S3_SYM = 1209,
     QUALIFY_SYM = 1210,
-    KEYWORD_USED_AS_IDENT = 1212,
-    KEYWORD_USED_AS_KEYWORD = 1213,
-    CONDITIONLESS_JOIN = 1214,
-    SUBQUERY_AS_EXPR = 1215,
-    EMPTY_FROM_CLAUSE = 1216
+    AUTO_SYM = 1211,
+    MANUAL_SYM = 1212,
+    BERNOULLI_SYM = 1213,
+    TABLESAMPLE_SYM = 1214,
+    KEYWORD_USED_AS_IDENT = 1216,
+    KEYWORD_USED_AS_KEYWORD = 1217,
+    CONDITIONLESS_JOIN = 1218,
+    PREFER_PARENTHESES = 1219,
+    EMPTY_FROM_CLAUSE = 1220
   };
 #endif
 /* Tokens.  */
@@ -1155,29 +1159,29 @@ extern int my_sql_parser_debug;
 #define LOOP_SYM 547
 #define LOW_PRIORITY 548
 #define LT 549
-#define MASTER_AUTO_POSITION_SYM 550
-#define MASTER_BIND_SYM 551
-#define MASTER_CONNECT_RETRY_SYM 552
-#define MASTER_DELAY_SYM 553
-#define MASTER_HOST_SYM 554
-#define MASTER_LOG_FILE_SYM 555
-#define MASTER_LOG_POS_SYM 556
-#define MASTER_PASSWORD_SYM 557
-#define MASTER_PORT_SYM 558
-#define MASTER_RETRY_COUNT_SYM 559
-#define MASTER_SSL_CAPATH_SYM 561
-#define MASTER_TLS_VERSION_SYM 562
-#define MASTER_SSL_CA_SYM 563
-#define MASTER_SSL_CERT_SYM 564
-#define MASTER_SSL_CIPHER_SYM 565
-#define MASTER_SSL_CRL_SYM 566
-#define MASTER_SSL_CRLPATH_SYM 567
-#define MASTER_SSL_KEY_SYM 568
-#define MASTER_SSL_SYM 569
-#define MASTER_SSL_VERIFY_SERVER_CERT_SYM 570
+#define OBSOLETE_TOKEN_550 550
+#define OBSOLETE_TOKEN_551 551
+#define OBSOLETE_TOKEN_552 552
+#define OBSOLETE_TOKEN_553 553
+#define OBSOLETE_TOKEN_554 554
+#define OBSOLETE_TOKEN_555 555
+#define OBSOLETE_TOKEN_556 556
+#define OBSOLETE_TOKEN_557 557
+#define OBSOLETE_TOKEN_558 558
+#define OBSOLETE_TOKEN_559 559
+#define OBSOLETE_TOKEN_561 561
+#define OBSOLETE_TOKEN_562 562
+#define OBSOLETE_TOKEN_563 563
+#define OBSOLETE_TOKEN_564 564
+#define OBSOLETE_TOKEN_565 565
+#define OBSOLETE_TOKEN_566 566
+#define OBSOLETE_TOKEN_567 567
+#define OBSOLETE_TOKEN_568 568
+#define OBSOLETE_TOKEN_569 569
+#define OBSOLETE_TOKEN_570 570
 #define MASTER_SYM 571
-#define MASTER_USER_SYM 572
-#define MASTER_HEARTBEAT_PERIOD_SYM 573
+#define OBSOLETE_TOKEN_572 572
+#define OBSOLETE_TOKEN_573 573
 #define MATCH 574
 #define MAX_CONNECTIONS_PER_HOUR 575
 #define MAX_QUERIES_PER_HOUR 576
@@ -1570,8 +1574,8 @@ extern int my_sql_parser_debug;
 #define RESOURCE_SYM 963
 #define SYSTEM_SYM 964
 #define VCPU_SYM 965
-#define MASTER_PUBLIC_KEY_PATH_SYM 966
-#define GET_MASTER_PUBLIC_KEY_SYM 967
+#define OBSOLETE_TOKEN_966 966
+#define OBSOLETE_TOKEN_967 967
 #define RESTART_SYM 968
 #define DEFINITION_SYM 969
 #define DESCRIPTION_SYM 970
@@ -1593,10 +1597,10 @@ extern int my_sql_parser_debug;
 #define OJ_SYM 986
 #define NETWORK_NAMESPACE_SYM 987
 #define RANDOM_SYM 988
-#define MASTER_COMPRESSION_ALGORITHM_SYM 989
-#define MASTER_ZSTD_COMPRESSION_LEVEL_SYM 990
+#define OBSOLETE_TOKEN_989 989
+#define OBSOLETE_TOKEN_990 990
 #define PRIVILEGE_CHECKS_USER_SYM 991
-#define MASTER_TLS_CIPHERSUITES_SYM 992
+#define OBSOLETE_TOKEN_992 992
 #define REQUIRE_ROW_FORMAT_SYM 993
 #define PASSWORD_LOCK_TIME_SYM 994
 #define FAILED_LOGIN_ATTEMPTS_SYM 995
@@ -1665,11 +1669,15 @@ extern int my_sql_parser_debug;
 #define PARALLEL_SYM 1208
 #define S3_SYM 1209
 #define QUALIFY_SYM 1210
-#define KEYWORD_USED_AS_IDENT 1212
-#define KEYWORD_USED_AS_KEYWORD 1213
-#define CONDITIONLESS_JOIN 1214
-#define SUBQUERY_AS_EXPR 1215
-#define EMPTY_FROM_CLAUSE 1216
+#define AUTO_SYM 1211
+#define MANUAL_SYM 1212
+#define BERNOULLI_SYM 1213
+#define TABLESAMPLE_SYM 1214
+#define KEYWORD_USED_AS_IDENT 1216
+#define KEYWORD_USED_AS_KEYWORD 1217
+#define CONDITIONLESS_JOIN 1218
+#define PREFER_PARENTHESES 1219
+#define EMPTY_FROM_CLAUSE 1220
 
 /* Value type.  */
 
@@ -1691,4 +1699,4 @@ struct MY_SQL_PARSER_LTYPE
 
 int my_sql_parser_parse (class THD *YYTHD, class Parse_tree_root **parse_tree);
 
-#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_13732547_1702565708_68_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_15014491_1712730762_01_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
