@@ -1,13 +1,21 @@
-/* Copyright (c) 2004, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2004, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  it under the terms of the GNU General Public License, version 2.0,
+  as published by the Free Software Foundation.
+
+  This program is designed to work with certain software (including
+  but not limited to OpenSSL) that is licensed under separate terms,
+  as designated in a particular file or component or in included license
+  documentation.  The authors of MySQL hereby grant you an additional
+  permission to link the program and your derivative works with the
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  GNU General Public License, version 2.0, for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -32,7 +40,7 @@
   --with-example-storage-engine
 
   Once this is done, MySQL will let you create tables with:<br>
-  CREATE TABLE <table name> (...) ENGINE=EXAMPLE;
+  CREATE TABLE \<table name\> (...) ENGINE=EXAMPLE;
 
   The example storage engine is set up to use table locks. It
   implements an example "SHARE" that is inserted into a hash by table
@@ -56,7 +64,6 @@
   ha_example::info
   ha_example::rnd_init
   ha_example::extra
-  ENUM HA_EXTRA_CACHE        Cache record in HA_rrnd()
   ha_example::rnd_next
   ha_example::rnd_next
   ha_example::rnd_next
@@ -67,7 +74,6 @@
   ha_example::rnd_next
   ha_example::rnd_next
   ha_example::extra
-  ENUM HA_EXTRA_NO_CACHE     End caching of records (def)
   ha_example::external_lock
   ha_example::extra
   ENUM HA_EXTRA_RESET        Reset database to after open
@@ -77,9 +83,9 @@
   rnd_next signals that it has reached the end of its data. Also note that
   the table in question was already opened; had it not been open, a call to
   ha_example::open() would also have been necessary. Calls to
-  ha_example::extra() are hints as to what will be occuring to the request.
+  ha_example::extra() are hints as to what will be occurring to the request.
 
-  A Longer Example can be found called the "Skeleton Engine" which can be 
+  A Longer Example can be found called the "Skeleton Engine" which can be
   found on TangentOrg. It has both an engine and a full build environment
   for building a pluggable storage engine.
 
@@ -87,44 +93,47 @@
     -Brian
 */
 
-#include "sql_class.h"           // MYSQL_HANDLERTON_INTERFACE_VERSION
-#include "ha_example.h"
-#include "probes_mysql.h"
-#include "sql_plugin.h"
+#include "storage/example/ha_example.h"
 
-static handler *example_create_handler(handlerton *hton,
-                                       TABLE_SHARE *table, 
-                                       MEM_ROOT *mem_root);
+#include "my_dbug.h"
+#include "mysql/plugin.h"
+#include "nulls.h"
+#include "sql/sql_class.h"
+#include "sql/sql_plugin.h"
+#include "typelib.h"
+
+static handler *example_create_handler(handlerton *hton, TABLE_SHARE *table,
+                                       bool partitioned, MEM_ROOT *mem_root);
 
 handlerton *example_hton;
 
 /* Interface to mysqld, to check system tables supported by SE */
-static const char* example_system_database();
 static bool example_is_supported_system_table(const char *db,
-                                      const char *table_name,
-                                      bool is_sql_layer_system_table);
+                                              const char *table_name,
+                                              bool is_sql_layer_system_table);
 
-Example_share::Example_share()
-{
-  thr_lock_init(&lock);
+Example_share::Example_share() { thr_lock_init(&lock); }
+
+static int example_init_func(void *p) {
+  DBUG_TRACE;
+
+  example_hton = (handlerton *)p;
+  example_hton->state = SHOW_OPTION_YES;
+  example_hton->create = example_create_handler;
+  example_hton->flags = HTON_CAN_RECREATE;
+  example_hton->is_supported_system_table = example_is_supported_system_table;
+
+  return 0;
 }
 
+static int example_deinit_func(void *p [[maybe_unused]]) {
+  DBUG_TRACE;
 
-static int example_init_func(void *p)
-{
-  DBUG_ENTER("example_init_func");
+  assert(p);
 
-  example_hton= (handlerton *)p;
-  example_hton->state=                     SHOW_OPTION_YES;
-  example_hton->create=                    example_create_handler;
-  example_hton->flags=                     HTON_CAN_RECREATE;
-  example_hton->system_database=   example_system_database;
-  example_hton->is_supported_system_table= example_is_supported_system_table;
-
-  DBUG_RETURN(0);
+  return 0;
 }
 
-
 /**
   @brief
   Example of simple lock controls. The "share" it creates is a
@@ -133,76 +142,30 @@ static int example_init_func(void *p)
   they are needed to function.
 */
 
-Example_share *ha_example::get_share()
-{
+Example_share *ha_example::get_share() {
   Example_share *tmp_share;
 
-  DBUG_ENTER("ha_example::get_share()");
+  DBUG_TRACE;
 
   lock_shared_ha_data();
-  if (!(tmp_share= static_cast<Example_share*>(get_ha_share_ptr())))
-  {
-    tmp_share= new Example_share;
-    if (!tmp_share)
-      goto err;
+  if (!(tmp_share = static_cast<Example_share *>(get_ha_share_ptr()))) {
+    tmp_share = new Example_share;
+    if (!tmp_share) goto err;
 
-    set_ha_share_ptr(static_cast<Handler_share*>(tmp_share));
+    set_ha_share_ptr(static_cast<Handler_share *>(tmp_share));
   }
 err:
   unlock_shared_ha_data();
-  DBUG_RETURN(tmp_share);
+  return tmp_share;
 }
 
-
-static handler* example_create_handler(handlerton *hton,
-                                       TABLE_SHARE *table, 
-                                       MEM_ROOT *mem_root)
-{
+static handler *example_create_handler(handlerton *hton, TABLE_SHARE *table,
+                                       bool, MEM_ROOT *mem_root) {
   return new (mem_root) ha_example(hton, table);
 }
 
 ha_example::ha_example(handlerton *hton, TABLE_SHARE *table_arg)
-  :handler(hton, table_arg)
-{}
-
-
-/**
-  @brief
-  If frm_error() is called then we will use this to determine
-  the file extensions that exist for the storage engine. This is also
-  used by the default rename_table and delete_table method in
-  handler.cc.
-
-  For engines that have two file name extentions (separate meta/index file
-  and data file), the order of elements is relevant. First element of engine
-  file name extentions array should be meta/index file extention. Second
-  element - data file extention. This order is assumed by
-  prepare_for_repair() when REPAIR TABLE ... USE_FRM is issued.
-
-  @see
-  rename_table method in handler.cc and
-  delete_table method in handler.cc
-*/
-
-static const char *ha_example_exts[] = {
-  NullS
-};
-
-const char **ha_example::bas_ext() const
-{
-  return ha_example_exts;
-}
-
-/*
-  Following handler function provides access to
-  system database specific to SE. This interface
-  is optional, so every SE need not implement it.
-*/
-const char* ha_example_system_database= NULL;
-const char* example_system_database()
-{
-  return ha_example_system_database;
-}
+    : handler(hton, table_arg) {}
 
 /*
   List of all system tables specific to the SE.
@@ -213,9 +176,8 @@ const char* example_system_database()
 
   This array is optional, so every SE need not implement it.
 */
-static st_system_tablename ha_example_system_tables[]= {
-  {(const char*)NULL, (const char*)NULL}
-};
+static st_handler_tablename ha_example_system_tables[] = {
+    {(const char *)nullptr, (const char *)nullptr}};
 
 /**
   @brief Check if the given db.tablename is a system table for this SE.
@@ -225,26 +187,21 @@ static st_system_tablename ha_example_system_tables[]= {
   @param is_sql_layer_system_table  if the supplied db.table_name is a SQL
                                     layer system table.
 
-  @return
-    @retval TRUE   Given db.table_name is supported system table.
-    @retval FALSE  Given db.table_name is not a supported system table.
+  @retval true   Given db.table_name is supported system table.
+  @retval false  Given db.table_name is not a supported system table.
 */
 static bool example_is_supported_system_table(const char *db,
                                               const char *table_name,
-                                              bool is_sql_layer_system_table)
-{
-  st_system_tablename *systab;
+                                              bool is_sql_layer_system_table) {
+  st_handler_tablename *systab;
 
   // Does this SE support "ALL" SQL layer system tables ?
-  if (is_sql_layer_system_table)
-    return false;
+  if (is_sql_layer_system_table) return false;
 
   // Check if this is SE layer system tables
-  systab= ha_example_system_tables;
-  while (systab && systab->db)
-  {
-    if (systab->db == db &&
-        strcmp(systab->tablename, table_name) == 0)
+  systab = ha_example_system_tables;
+  while (systab && systab->db) {
+    if (systab->db == db && strcmp(systab->tablename, table_name) == 0)
       return true;
     systab++;
   }
@@ -252,7 +209,6 @@ static bool example_is_supported_system_table(const char *db,
   return false;
 }
 
-
 /**
   @brief
   Used for opening tables. The name will be the name of the file.
@@ -269,18 +225,15 @@ static bool example_is_supported_system_table(const char *db,
   handler::ha_open() in handler.cc
 */
 
-int ha_example::open(const char *name, int mode, uint test_if_locked)
-{
-  DBUG_ENTER("ha_example::open");
+int ha_example::open(const char *, int, uint, const dd::Table *) {
+  DBUG_TRACE;
 
-  if (!(share = get_share()))
-    DBUG_RETURN(1);
-  thr_lock_data_init(&share->lock,&lock,NULL);
+  if (!(share = get_share())) return 1;
+  thr_lock_data_init(&share->lock, &lock, nullptr);
 
-  DBUG_RETURN(0);
+  return 0;
 }
 
-
 /**
   @brief
   Closes a table.
@@ -296,13 +249,11 @@ int ha_example::open(const char *name, int mode, uint test_if_locked)
   sql_base.cc, sql_select.cc and table.cc
 */
 
-int ha_example::close(void)
-{
-  DBUG_ENTER("ha_example::close");
-  DBUG_RETURN(0);
+int ha_example::close(void) {
+  DBUG_TRACE;
+  return 0;
 }
 
-
 /**
   @brief
   write_row() inserts a row. No extra() hint is given currently if a bulk load
@@ -333,19 +284,17 @@ int ha_example::close(void)
   sql_insert.cc, sql_select.cc, sql_table.cc, sql_udf.cc and sql_update.cc
 */
 
-int ha_example::write_row(uchar *buf)
-{
-  DBUG_ENTER("ha_example::write_row");
+int ha_example::write_row(uchar *) {
+  DBUG_TRACE;
   /*
     Example of a successful write_row. We don't store the data
     anywhere; they are thrown away. A real implementation will
     probably need to do something with 'buf'. We report a success
     here, to pretend that the insert was successful.
   */
-  DBUG_RETURN(0);
+  return 0;
 }
 
-
 /**
   @brief
   Yes, update_row() does what you expect, it updates a row. old_data will have
@@ -369,14 +318,11 @@ int ha_example::write_row(uchar *buf)
   @see
   sql_select.cc, sql_acl.cc, sql_update.cc and sql_insert.cc
 */
-int ha_example::update_row(const uchar *old_data, uchar *new_data)
-{
-
-  DBUG_ENTER("ha_example::update_row");
-  DBUG_RETURN(HA_ERR_WRONG_COMMAND);
+int ha_example::update_row(const uchar *, uchar *) {
+  DBUG_TRACE;
+  return HA_ERR_WRONG_COMMAND;
 }
 
-
 /**
   @brief
   This will delete a row. buf will contain a copy of the row to be deleted.
@@ -397,13 +343,11 @@ int ha_example::update_row(const uchar *old_data, uchar *new_data)
   sql_acl.cc, sql_udf.cc, sql_delete.cc, sql_insert.cc and sql_select.cc
 */
 
-int ha_example::delete_row(const uchar *buf)
-{
-  DBUG_ENTER("ha_example::delete_row");
-  DBUG_RETURN(HA_ERR_WRONG_COMMAND);
+int ha_example::delete_row(const uchar *) {
+  DBUG_TRACE;
+  return HA_ERR_WRONG_COMMAND;
 }
 
-
 /**
   @brief
   Positions an index cursor to the index specified in the handle. Fetches the
@@ -411,52 +355,38 @@ int ha_example::delete_row(const uchar *buf)
   index.
 */
 
-int ha_example::index_read_map(uchar *buf, const uchar *key,
-                               key_part_map keypart_map __attribute__((unused)),
-                               enum ha_rkey_function find_flag
-                               __attribute__((unused)))
-{
+int ha_example::index_read_map(uchar *, const uchar *, key_part_map,
+                               enum ha_rkey_function) {
   int rc;
-  DBUG_ENTER("ha_example::index_read");
-  MYSQL_INDEX_READ_ROW_START(table_share->db.str, table_share->table_name.str);
-  rc= HA_ERR_WRONG_COMMAND;
-  MYSQL_INDEX_READ_ROW_DONE(rc);
-  DBUG_RETURN(rc);
+  DBUG_TRACE;
+  rc = HA_ERR_WRONG_COMMAND;
+  return rc;
 }
 
-
 /**
   @brief
   Used to read forward through the index.
 */
 
-int ha_example::index_next(uchar *buf)
-{
+int ha_example::index_next(uchar *) {
   int rc;
-  DBUG_ENTER("ha_example::index_next");
-  MYSQL_INDEX_READ_ROW_START(table_share->db.str, table_share->table_name.str);
-  rc= HA_ERR_WRONG_COMMAND;
-  MYSQL_INDEX_READ_ROW_DONE(rc);
-  DBUG_RETURN(rc);
+  DBUG_TRACE;
+  rc = HA_ERR_WRONG_COMMAND;
+  return rc;
 }
 
-
 /**
   @brief
   Used to read backwards through the index.
 */
 
-int ha_example::index_prev(uchar *buf)
-{
+int ha_example::index_prev(uchar *) {
   int rc;
-  DBUG_ENTER("ha_example::index_prev");
-  MYSQL_INDEX_READ_ROW_START(table_share->db.str, table_share->table_name.str);
-  rc= HA_ERR_WRONG_COMMAND;
-  MYSQL_INDEX_READ_ROW_DONE(rc);
-  DBUG_RETURN(rc);
+  DBUG_TRACE;
+  rc = HA_ERR_WRONG_COMMAND;
+  return rc;
 }
 
-
 /**
   @brief
   index_first() asks for the first key in the index.
@@ -467,17 +397,13 @@ int ha_example::index_prev(uchar *buf)
   @see
   opt_range.cc, opt_sum.cc, sql_handler.cc and sql_select.cc
 */
-int ha_example::index_first(uchar *buf)
-{
+int ha_example::index_first(uchar *) {
   int rc;
-  DBUG_ENTER("ha_example::index_first");
-  MYSQL_INDEX_READ_ROW_START(table_share->db.str, table_share->table_name.str);
-  rc= HA_ERR_WRONG_COMMAND;
-  MYSQL_INDEX_READ_ROW_DONE(rc);
-  DBUG_RETURN(rc);
+  DBUG_TRACE;
+  rc = HA_ERR_WRONG_COMMAND;
+  return rc;
 }
 
-
 /**
   @brief
   index_last() asks for the last key in the index.
@@ -488,17 +414,13 @@ int ha_example::index_first(uchar *buf)
   @see
   opt_range.cc, opt_sum.cc, sql_handler.cc and sql_select.cc
 */
-int ha_example::index_last(uchar *buf)
-{
+int ha_example::index_last(uchar *) {
   int rc;
-  DBUG_ENTER("ha_example::index_last");
-  MYSQL_INDEX_READ_ROW_START(table_share->db.str, table_share->table_name.str);
-  rc= HA_ERR_WRONG_COMMAND;
-  MYSQL_INDEX_READ_ROW_DONE(rc);
-  DBUG_RETURN(rc);
+  DBUG_TRACE;
+  rc = HA_ERR_WRONG_COMMAND;
+  return rc;
 }
 
-
 /**
   @brief
   rnd_init() is called when the system wants the storage engine to do a table
@@ -506,25 +428,23 @@ int ha_example::index_last(uchar *buf)
   rnd_init() is called.
 
   @details
-  Called from filesort.cc, records.cc, sql_handler.cc, sql_select.cc, sql_table.cc,
-  and sql_update.cc.
+  Called from filesort.cc, records.cc, sql_handler.cc, sql_select.cc,
+  sql_table.cc, and sql_update.cc.
 
   @see
-  filesort.cc, records.cc, sql_handler.cc, sql_select.cc, sql_table.cc and sql_update.cc
+  filesort.cc, records.cc, sql_handler.cc, sql_select.cc, sql_table.cc and
+  sql_update.cc
 */
-int ha_example::rnd_init(bool scan)
-{
-  DBUG_ENTER("ha_example::rnd_init");
-  DBUG_RETURN(0);
+int ha_example::rnd_init(bool) {
+  DBUG_TRACE;
+  return 0;
 }
 
-int ha_example::rnd_end()
-{
-  DBUG_ENTER("ha_example::rnd_end");
-  DBUG_RETURN(0);
+int ha_example::rnd_end() {
+  DBUG_TRACE;
+  return 0;
 }
 
-
 /**
   @brief
   This is called for each row of the table scan. When you run out of records
@@ -533,24 +453,20 @@ int ha_example::rnd_end()
   in a manner that will allow the server to understand it.
 
   @details
-  Called from filesort.cc, records.cc, sql_handler.cc, sql_select.cc, sql_table.cc,
-  and sql_update.cc.
+  Called from filesort.cc, records.cc, sql_handler.cc, sql_select.cc,
+  sql_table.cc, and sql_update.cc.
 
   @see
-  filesort.cc, records.cc, sql_handler.cc, sql_select.cc, sql_table.cc and sql_update.cc
+  filesort.cc, records.cc, sql_handler.cc, sql_select.cc, sql_table.cc and
+  sql_update.cc
 */
-int ha_example::rnd_next(uchar *buf)
-{
+int ha_example::rnd_next(uchar *) {
   int rc;
-  DBUG_ENTER("ha_example::rnd_next");
-  MYSQL_READ_ROW_START(table_share->db.str, table_share->table_name.str,
-                       TRUE);
-  rc= HA_ERR_END_OF_FILE;
-  MYSQL_READ_ROW_DONE(rc);
-  DBUG_RETURN(rc);
+  DBUG_TRACE;
+  rc = HA_ERR_END_OF_FILE;
+  return rc;
 }
 
-
 /**
   @brief
   position() is called after each call to rnd_next() if the data needs
@@ -572,12 +488,7 @@ int ha_example::rnd_next(uchar *buf)
   @see
   filesort.cc, sql_select.cc, sql_delete.cc and sql_update.cc
 */
-void ha_example::position(const uchar *record)
-{
-  DBUG_ENTER("ha_example::position");
-  DBUG_VOID_RETURN;
-}
-
+void ha_example::position(const uchar *) { DBUG_TRACE; }
 
 /**
   @brief
@@ -587,31 +498,27 @@ void ha_example::position(const uchar *record)
   or position you saved when position() was called.
 
   @details
-  Called from filesort.cc, records.cc, sql_insert.cc, sql_select.cc, and sql_update.cc.
+  Called from filesort.cc, records.cc, sql_insert.cc, sql_select.cc, and
+  sql_update.cc.
 
   @see
   filesort.cc, records.cc, sql_insert.cc, sql_select.cc and sql_update.cc
 */
-int ha_example::rnd_pos(uchar *buf, uchar *pos)
-{
+int ha_example::rnd_pos(uchar *, uchar *) {
   int rc;
-  DBUG_ENTER("ha_example::rnd_pos");
-  MYSQL_READ_ROW_START(table_share->db.str, table_share->table_name.str,
-                       TRUE);
-  rc= HA_ERR_WRONG_COMMAND;
-  MYSQL_READ_ROW_DONE(rc);
-  DBUG_RETURN(rc);
+  DBUG_TRACE;
+  rc = HA_ERR_WRONG_COMMAND;
+  return rc;
 }
 
-
 /**
   @brief
   ::info() is used to return information to the optimizer. See my_base.h for
   the complete description.
 
   @details
-  Currently this table handler doesn't implement most of the fields really needed.
-  SHOW also makes use of this data.
+  Currently this table handler doesn't implement most of the fields really
+  needed. SHOW also makes use of this data.
 
   You will probably want to have the following in your code:
   @code
@@ -633,22 +540,20 @@ int ha_example::rnd_pos(uchar *buf, uchar *pos)
 
   Called in filesort.cc, ha_heap.cc, item_sum.cc, opt_sum.cc, sql_delete.cc,
   sql_delete.cc, sql_derived.cc, sql_select.cc, sql_select.cc, sql_select.cc,
-  sql_select.cc, sql_select.cc, sql_show.cc, sql_show.cc, sql_show.cc, sql_show.cc,
-  sql_table.cc, sql_union.cc, and sql_update.cc.
+  sql_select.cc, sql_select.cc, sql_show.cc, sql_show.cc, sql_show.cc,
+  sql_show.cc, sql_table.cc, sql_union.cc, and sql_update.cc.
 
   @see
-  filesort.cc, ha_heap.cc, item_sum.cc, opt_sum.cc, sql_delete.cc, sql_delete.cc,
-  sql_derived.cc, sql_select.cc, sql_select.cc, sql_select.cc, sql_select.cc,
-  sql_select.cc, sql_show.cc, sql_show.cc, sql_show.cc, sql_show.cc, sql_table.cc,
-  sql_union.cc and sql_update.cc
+  filesort.cc, ha_heap.cc, item_sum.cc, opt_sum.cc, sql_delete.cc,
+  sql_delete.cc, sql_derived.cc, sql_select.cc, sql_select.cc, sql_select.cc,
+  sql_select.cc, sql_select.cc, sql_show.cc, sql_show.cc, sql_show.cc,
+  sql_show.cc, sql_table.cc, sql_union.cc and sql_update.cc
 */
-int ha_example::info(uint flag)
-{
-  DBUG_ENTER("ha_example::info");
-  DBUG_RETURN(0);
+int ha_example::info(uint) {
+  DBUG_TRACE;
+  return 0;
 }
 
-
 /**
   @brief
   extra() is called whenever the server wishes to send a hint to
@@ -658,69 +563,43 @@ int ha_example::info(uint flag)
     @see
   ha_innodb.cc
 */
-int ha_example::extra(enum ha_extra_function operation)
-{
-  DBUG_ENTER("ha_example::extra");
-  DBUG_RETURN(0);
+int ha_example::extra(enum ha_extra_function) {
+  DBUG_TRACE;
+  return 0;
 }
 
-
 /**
   @brief
-  Used to delete all rows in a table, including cases of truncate and cases where
-  the optimizer realizes that all rows will be removed as a result of an SQL statement.
+  Used to delete all rows in a table, including cases of truncate and cases
+  where the optimizer realizes that all rows will be removed as a result of an
+  SQL statement.
 
   @details
   Called from item_sum.cc by Item_func_group_concat::clear(),
   Item_sum_count_distinct::clear(), and Item_func_group_concat::clear().
   Called from sql_delete.cc by mysql_delete().
   Called from sql_select.cc by JOIN::reinit().
-  Called from sql_union.cc by st_select_lex_unit::exec().
+  Called from sql_union.cc by st_query_block_query_expression::exec().
 
   @see
   Item_func_group_concat::clear(), Item_sum_count_distinct::clear() and
   Item_func_group_concat::clear() in item_sum.cc;
   mysql_delete() in sql_delete.cc;
   JOIN::reinit() in sql_select.cc and
-  st_select_lex_unit::exec() in sql_union.cc.
+  st_query_block_query_expression::exec() in sql_union.cc.
 */
-int ha_example::delete_all_rows()
-{
-  DBUG_ENTER("ha_example::delete_all_rows");
-  DBUG_RETURN(HA_ERR_WRONG_COMMAND);
+int ha_example::delete_all_rows() {
+  DBUG_TRACE;
+  return HA_ERR_WRONG_COMMAND;
 }
 
-
-/**
-  @brief
-  Used for handler specific truncate table.  The table is locked in
-  exclusive mode and handler is responsible for reseting the auto-
-  increment counter.
-
-  @details
-  Called from Truncate_statement::handler_truncate.
-  Not used if the handlerton supports HTON_CAN_RECREATE, unless this
-  engine can be used as a partition. In this case, it is invoked when
-  a particular partition is to be truncated.
-
-  @see
-  Truncate_statement in sql_truncate.cc
-  Remarks in handler::truncate.
-*/
-int ha_example::truncate()
-{
-  DBUG_ENTER("ha_example::truncate");
-  DBUG_RETURN(HA_ERR_WRONG_COMMAND);
-}
-
-
 /**
   @brief
   This create a lock on the table. If you are implementing a storage engine
   that can handle transacations look at ha_berkely.cc to see how you will
   want to go about doing this. Otherwise you should consider calling flock()
-  here. Hint: Read the section "locking functions for mysql" in lock.cc to understand
-  this.
+  here. Hint: Read the section "locking functions for mysql" in lock.cc to
+  understand this.
 
   @details
   Called from lock.cc by lock_external() and unlock_external(). Also called
@@ -731,13 +610,11 @@ int ha_example::truncate()
   the section "locking functions for mysql" in lock.cc;
   copy_data_between_tables() in sql_table.cc.
 */
-int ha_example::external_lock(THD *thd, int lock_type)
-{
-  DBUG_ENTER("ha_example::external_lock");
-  DBUG_RETURN(0);
+int ha_example::external_lock(THD *, int) {
+  DBUG_TRACE;
+  return 0;
 }
 
-
 /**
   @brief
   The idea with handler::store_lock() is: The statement decides which locks
@@ -775,17 +652,13 @@ int ha_example::external_lock(THD *thd, int lock_type)
   @see
   get_lock_data() in lock.cc
 */
-THR_LOCK_DATA **ha_example::store_lock(THD *thd,
-                                       THR_LOCK_DATA **to,
-                                       enum thr_lock_type lock_type)
-{
-  if (lock_type != TL_IGNORE && lock.type == TL_UNLOCK)
-    lock.type=lock_type;
-  *to++= &lock;
+THR_LOCK_DATA **ha_example::store_lock(THD *, THR_LOCK_DATA **to,
+                                       enum thr_lock_type lock_type) {
+  if (lock_type != TL_IGNORE && lock.type == TL_UNLOCK) lock.type = lock_type;
+  *to++ = &lock;
   return to;
 }
 
-
 /**
   @brief
   Used to delete a table. By the time delete_table() has been called all
@@ -795,8 +668,8 @@ THR_LOCK_DATA **ha_example::store_lock(THD *thd,
 
   @details
   If you do not implement this, the default delete_table() is called from
-  handler.cc and it will delete all files with the file extensions returned
-  by bas_ext().
+  handler.cc and it will delete all files with the file extensions from
+  handlerton::file_extensions.
 
   Called from handler.cc by delete_table and ha_create_table(). Only used
   during create if the table_flag HA_DROP_BEFORE_CREATE was specified for
@@ -805,35 +678,32 @@ THR_LOCK_DATA **ha_example::store_lock(THD *thd,
   @see
   delete_table and ha_create_table() in handler.cc
 */
-int ha_example::delete_table(const char *name)
-{
-  DBUG_ENTER("ha_example::delete_table");
+int ha_example::delete_table(const char *, const dd::Table *) {
+  DBUG_TRACE;
   /* This is not implemented but we want someone to be able that it works. */
-  DBUG_RETURN(0);
+  return 0;
 }
 
-
 /**
   @brief
   Renames a table from one name to another via an alter table call.
 
   @details
   If you do not implement this, the default rename_table() is called from
-  handler.cc and it will delete all files with the file extensions returned
-  by bas_ext().
+  handler.cc and it will delete all files with the file extensions from
+  handlerton::file_extensions.
 
   Called from sql_table.cc by mysql_rename_table().
 
   @see
   mysql_rename_table() in sql_table.cc
 */
-int ha_example::rename_table(const char * from, const char * to)
-{
-  DBUG_ENTER("ha_example::rename_table ");
-  DBUG_RETURN(HA_ERR_WRONG_COMMAND);
+int ha_example::rename_table(const char *, const char *, const dd::Table *,
+                             dd::Table *) {
+  DBUG_TRACE;
+  return HA_ERR_WRONG_COMMAND;
 }
 
-
 /**
   @brief
   Given a starting key and an ending key, estimate the number of rows that
@@ -847,13 +717,16 @@ int ha_example::rename_table(const char * from, const char * to)
   @see
   check_quick_keys() in opt_range.cc
 */
-ha_rows ha_example::records_in_range(uint inx, key_range *min_key,
-                                     key_range *max_key)
-{
-  DBUG_ENTER("ha_example::records_in_range");
-  DBUG_RETURN(10);                         // low number to force index usage
+ha_rows ha_example::records_in_range(uint, key_range *, key_range *) {
+  DBUG_TRACE;
+  return 10;  // low number to force index usage
 }
 
+static MYSQL_THDVAR_STR(last_create_thdvar, PLUGIN_VAR_MEMALLOC, nullptr,
+                        nullptr, nullptr, nullptr);
+
+static MYSQL_THDVAR_UINT(create_count_thdvar, 0, nullptr, nullptr, nullptr, 0,
+                         0, 1000, 0);
 
 /**
   @brief
@@ -874,152 +747,166 @@ ha_rows ha_example::records_in_range(uint inx, key_range *min_key,
   ha_create_table() in handle.cc
 */
 
-int ha_example::create(const char *name, TABLE *table_arg,
-                       HA_CREATE_INFO *create_info)
-{
-  DBUG_ENTER("ha_example::create");
+int ha_example::create(const char *name, TABLE *, HA_CREATE_INFO *,
+                       dd::Table *) {
+  DBUG_TRACE;
   /*
     This is not implemented but we want someone to be able to see that it
     works.
   */
-  DBUG_RETURN(0);
-}
 
+  /*
+    It's just an example of THDVAR_SET() usage below.
+  */
+  THD *thd = ha_thd();
+  char *buf = (char *)my_malloc(PSI_NOT_INSTRUMENTED, SHOW_VAR_FUNC_BUFF_SIZE,
+                                MYF(MY_FAE));
+  snprintf(buf, SHOW_VAR_FUNC_BUFF_SIZE, "Last creation '%s'", name);
+  THDVAR_SET(thd, last_create_thdvar, buf);
+  my_free(buf);
 
-struct st_mysql_storage_engine example_storage_engine=
-{ MYSQL_HANDLERTON_INTERFACE_VERSION };
+  uint count = THDVAR(thd, create_count_thdvar) + 1;
+  THDVAR_SET(thd, create_count_thdvar, &count);
 
-static ulong srv_enum_var= 0;
-static ulong srv_ulong_var= 0;
-static double srv_double_var= 0;
-
-const char *enum_var_names[]=
-{
-  "e1", "e2", NullS
-};
-
-TYPELIB enum_var_typelib=
-{
-  array_elements(enum_var_names) - 1, "enum_var_typelib",
-  enum_var_names, NULL
-};
-
-static MYSQL_SYSVAR_ENUM(
-  enum_var,                       // name
-  srv_enum_var,                   // varname
-  PLUGIN_VAR_RQCMDARG,            // opt
-  "Sample ENUM system variable.", // comment
-  NULL,                           // check
-  NULL,                           // update
-  0,                              // def
-  &enum_var_typelib);             // typelib
-
-static MYSQL_SYSVAR_ULONG(
-  ulong_var,
-  srv_ulong_var,
-  PLUGIN_VAR_RQCMDARG,
-  "0..1000",
-  NULL,
-  NULL,
-  8,
-  0,
-  1000,
-  0);
-
-static MYSQL_SYSVAR_DOUBLE(
-  double_var,
-  srv_double_var,
-  PLUGIN_VAR_RQCMDARG,
-  "0.500000..1000.500000",
-  NULL,
-  NULL,
-  8.5,
-  0.5,
-  1000.5,
-  0);                             // reserved always 0
-
-static MYSQL_THDVAR_DOUBLE(
-  double_thdvar,
-  PLUGIN_VAR_RQCMDARG,
-  "0.500000..1000.500000",
-  NULL,
-  NULL,
-  8.5,
-  0.5,
-  1000.5,
-  0);
-
-static struct st_mysql_sys_var* example_system_variables[]= {
-  MYSQL_SYSVAR(enum_var),
-  MYSQL_SYSVAR(ulong_var),
-  MYSQL_SYSVAR(double_var),
-  MYSQL_SYSVAR(double_thdvar),
-  NULL
-};
-
-// this is an example of SHOW_FUNC and of my_snprintf() service
-static int show_func_example(MYSQL_THD thd, struct st_mysql_show_var *var,
-                             char *buf)
-{
-  var->type= SHOW_CHAR;
-  var->value= buf; // it's of SHOW_VAR_FUNC_BUFF_SIZE bytes
-  my_snprintf(buf, SHOW_VAR_FUNC_BUFF_SIZE,
-              "enum_var is %lu, ulong_var is %lu, "
-              "double_var is %f, %.6b", // %b is a MySQL extension
-              srv_enum_var, srv_ulong_var, srv_double_var, "really");
   return 0;
 }
 
-struct example_vars_t
-{
-	ulong  var1;
-	double var2;
-	char   var3[64];
-  bool   var4;
-  bool   var5;
-  ulong  var6;
-};
+struct st_mysql_storage_engine example_storage_engine = {
+    MYSQL_HANDLERTON_INTERFACE_VERSION};
 
-example_vars_t example_vars= {100, 20.01, "three hundred", true, 0, 8250};
+static ulong srv_enum_var = 0;
+static ulong srv_ulong_var = 0;
+static double srv_double_var = 0;
+static int srv_signed_int_var = 0;
+static long srv_signed_long_var = 0;
+static longlong srv_signed_longlong_var = 0;
 
-static st_mysql_show_var show_status_example[]=
-{
-  {"var1", (char *)&example_vars.var1, SHOW_LONG, SHOW_SCOPE_GLOBAL},
-  {"var2", (char *)&example_vars.var2, SHOW_DOUBLE, SHOW_SCOPE_GLOBAL},
-  {0,0,SHOW_UNDEF, SHOW_SCOPE_UNDEF} // null terminator required
-};
+const char *enum_var_names[] = {"e1", "e2", NullS};
 
-static struct st_mysql_show_var show_array_example[]=
-{
-  {"array", (char *)show_status_example, SHOW_ARRAY, SHOW_SCOPE_GLOBAL},
-  {"var3", (char *)&example_vars.var3, SHOW_CHAR, SHOW_SCOPE_GLOBAL},
-  {"var4", (char *)&example_vars.var4, SHOW_BOOL, SHOW_SCOPE_GLOBAL},
-  {0,0,SHOW_UNDEF, SHOW_SCOPE_UNDEF}
-};
+TYPELIB enum_var_typelib = {array_elements(enum_var_names) - 1,
+                            "enum_var_typelib", enum_var_names, nullptr};
 
-static struct st_mysql_show_var func_status[]=
-{
-  {"example_func_example", (char *)show_func_example, SHOW_FUNC, SHOW_SCOPE_GLOBAL},
-  {"example_status_var5", (char *)&example_vars.var5, SHOW_BOOL, SHOW_SCOPE_GLOBAL},
-  {"example_status_var6", (char *)&example_vars.var6, SHOW_LONG, SHOW_SCOPE_GLOBAL},
-  {"example_status",  (char *)show_array_example, SHOW_ARRAY, SHOW_SCOPE_GLOBAL},
-  {0,0,SHOW_UNDEF, SHOW_SCOPE_UNDEF}
-};
+static MYSQL_SYSVAR_ENUM(enum_var,                        // name
+                         srv_enum_var,                    // varname
+                         PLUGIN_VAR_RQCMDARG,             // opt
+                         "Sample ENUM system variable.",  // comment
+                         nullptr,                         // check
+                         nullptr,                         // update
+                         0,                               // def
+                         &enum_var_typelib);              // typelib
 
-mysql_declare_plugin(example)
-{
-  MYSQL_STORAGE_ENGINE_PLUGIN,
-  &example_storage_engine,
-  "EXAMPLE",
-  "Brian Aker, MySQL AB",
-  "Example storage engine",
-  PLUGIN_LICENSE_GPL,
-  example_init_func,                            /* Plugin Init */
-  NULL,                                         /* Plugin Deinit */
-  0x0001 /* 0.1 */,
-  func_status,                                  /* status variables */
-  example_system_variables,                     /* system variables */
-  NULL,                                         /* config options */
-  0,                                            /* flags */
+static MYSQL_SYSVAR_ULONG(ulong_var, srv_ulong_var, PLUGIN_VAR_RQCMDARG,
+                          "0..1000", nullptr, nullptr, 8, 0, 1000, 0);
+
+static MYSQL_SYSVAR_DOUBLE(double_var, srv_double_var, PLUGIN_VAR_RQCMDARG,
+                           "0.500000..1000.500000", nullptr, nullptr, 8.5, 0.5,
+                           1000.5,
+                           0);  // reserved always 0
+
+static MYSQL_THDVAR_DOUBLE(double_thdvar, PLUGIN_VAR_RQCMDARG,
+                           "0.500000..1000.500000", nullptr, nullptr, 8.5, 0.5,
+                           1000.5, 0);
+
+static MYSQL_SYSVAR_INT(signed_int_var, srv_signed_int_var, PLUGIN_VAR_RQCMDARG,
+                        "INT_MIN..INT_MAX", nullptr, nullptr, -10, INT_MIN,
+                        INT_MAX, 0);
+
+static MYSQL_THDVAR_INT(signed_int_thdvar, PLUGIN_VAR_RQCMDARG,
+                        "INT_MIN..INT_MAX", nullptr, nullptr, -10, INT_MIN,
+                        INT_MAX, 0);
+
+static MYSQL_SYSVAR_LONG(signed_long_var, srv_signed_long_var,
+                         PLUGIN_VAR_RQCMDARG, "LONG_MIN..LONG_MAX", nullptr,
+                         nullptr, -10, LONG_MIN, LONG_MAX, 0);
+
+static MYSQL_THDVAR_LONG(signed_long_thdvar, PLUGIN_VAR_RQCMDARG,
+                         "LONG_MIN..LONG_MAX", nullptr, nullptr, -10, LONG_MIN,
+                         LONG_MAX, 0);
+
+static MYSQL_SYSVAR_LONGLONG(signed_longlong_var, srv_signed_longlong_var,
+                             PLUGIN_VAR_RQCMDARG, "LLONG_MIN..LLONG_MAX",
+                             nullptr, nullptr, -10, LLONG_MIN, LLONG_MAX, 0);
+
+static MYSQL_THDVAR_LONGLONG(signed_longlong_thdvar, PLUGIN_VAR_RQCMDARG,
+                             "LLONG_MIN..LLONG_MAX", nullptr, nullptr, -10,
+                             LLONG_MIN, LLONG_MAX, 0);
+
+static SYS_VAR *example_system_variables[] = {
+    MYSQL_SYSVAR(enum_var),
+    MYSQL_SYSVAR(ulong_var),
+    MYSQL_SYSVAR(double_var),
+    MYSQL_SYSVAR(double_thdvar),
+    MYSQL_SYSVAR(last_create_thdvar),
+    MYSQL_SYSVAR(create_count_thdvar),
+    MYSQL_SYSVAR(signed_int_var),
+    MYSQL_SYSVAR(signed_int_thdvar),
+    MYSQL_SYSVAR(signed_long_var),
+    MYSQL_SYSVAR(signed_long_thdvar),
+    MYSQL_SYSVAR(signed_longlong_var),
+    MYSQL_SYSVAR(signed_longlong_thdvar),
+    nullptr};
+
+// this is an example of SHOW_FUNC
+static int show_func_example(MYSQL_THD, SHOW_VAR *var, char *buf) {
+  var->type = SHOW_CHAR;
+  var->value = buf;  // it's of SHOW_VAR_FUNC_BUFF_SIZE bytes
+  snprintf(buf, SHOW_VAR_FUNC_BUFF_SIZE,
+           "enum_var is %lu, ulong_var is %lu, "
+           "double_var is %f, signed_int_var is %d, "
+           "signed_long_var is %ld, signed_longlong_var is %lld",
+           srv_enum_var, srv_ulong_var, srv_double_var, srv_signed_int_var,
+           srv_signed_long_var, srv_signed_longlong_var);
+  return 0;
 }
-mysql_declare_plugin_end;
+
+struct example_vars_t {
+  ulong var1;
+  double var2;
+  char var3[64];
+  bool var4;
+  bool var5;
+  ulong var6;
+};
+
+example_vars_t example_vars = {100, 20.01, "three hundred", true, false, 8250};
+
+static SHOW_VAR show_status_example[] = {
+    {"var1", (char *)&example_vars.var1, SHOW_LONG, SHOW_SCOPE_GLOBAL},
+    {"var2", (char *)&example_vars.var2, SHOW_DOUBLE, SHOW_SCOPE_GLOBAL},
+    {nullptr, nullptr, SHOW_UNDEF,
+     SHOW_SCOPE_UNDEF}  // null terminator required
+};
+
+static SHOW_VAR show_array_example[] = {
+    {"array", (char *)show_status_example, SHOW_ARRAY, SHOW_SCOPE_GLOBAL},
+    {"var3", (char *)&example_vars.var3, SHOW_CHAR, SHOW_SCOPE_GLOBAL},
+    {"var4", (char *)&example_vars.var4, SHOW_BOOL, SHOW_SCOPE_GLOBAL},
+    {nullptr, nullptr, SHOW_UNDEF, SHOW_SCOPE_UNDEF}};
+
+static SHOW_VAR func_status[] = {
+    {"example_func_example", (char *)show_func_example, SHOW_FUNC,
+     SHOW_SCOPE_GLOBAL},
+    {"example_status_var5", (char *)&example_vars.var5, SHOW_BOOL,
+     SHOW_SCOPE_GLOBAL},
+    {"example_status_var6", (char *)&example_vars.var6, SHOW_LONG,
+     SHOW_SCOPE_GLOBAL},
+    {"example_status", (char *)show_array_example, SHOW_ARRAY,
+     SHOW_SCOPE_GLOBAL},
+    {nullptr, nullptr, SHOW_UNDEF, SHOW_SCOPE_UNDEF}};
+
+mysql_declare_plugin(example){
+    MYSQL_STORAGE_ENGINE_PLUGIN,
+    &example_storage_engine,
+    "EXAMPLE",
+    PLUGIN_AUTHOR_ORACLE,
+    "Example storage engine",
+    PLUGIN_LICENSE_GPL,
+    example_init_func,   /* Plugin Init */
+    nullptr,             /* Plugin check uninstall */
+    example_deinit_func, /* Plugin Deinit */
+    0x0001 /* 0.1 */,
+    func_status,              /* status variables */
+    example_system_variables, /* system variables */
+    nullptr,                  /* config options */
+    0,                        /* flags */
+} mysql_declare_plugin_end;
