@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -899,12 +900,12 @@ void dd_mdl_release(THD *thd, MDL_ticket **mdl);
 /** Returns thd associated with the trx or current_thd
 @param[in]      trx     transaction
 @return trx->mysql_thd or current_thd */
-THD *dd_thd_for_undo(const trx_t *trx);
+THD *dd_thd_for_undo(const trx_t &trx);
 
 /** Check if current undo needs a MDL or not
 @param[in]      trx     transaction
 @return true if MDL is necessary, otherwise false */
-bool dd_mdl_for_undo(const trx_t *trx);
+bool dd_mdl_for_undo(const trx_t &trx);
 
 /** Load foreign key constraint info for the dd::Table object.
 @param[out]     m_table         InnoDB table handle
