@@ -30,12 +30,12 @@
 #include <unordered_map>
 #include <vector>
 
-#include "storage/ndb/plugin/ndb_applier.h"
 #include "storage/ndb/plugin/ndb_share.h"
 #include "storage/ndb/plugin/ndb_thd.h"
 
 class THD;
 class Relay_log_info;
+class Ndb_applier;
 
 /*
   Class for ndbcluster thread specific data
@@ -43,17 +43,15 @@ class Relay_log_info;
 class Thd_ndb {
   THD *const m_thd;
 
-  Thd_ndb(THD *thd, const char *name);
+  Thd_ndb(THD *);
   ~Thd_ndb();
 
   uint32 options;
   uint32 trans_options;
   class Ndb_DDL_transaction_ctx *m_ddl_ctx;
-  // Thread name that owns this connection (usually the PFS thread name)
-  const char *const m_thread_name;
 
  public:
-  static Thd_ndb *seize(THD *thd, const char *name = nullptr);
+  static Thd_ndb *seize(THD *);
   static void release(Thd_ndb *thd_ndb);
 
   // Keeps track of stats for tables taking part in transaction
@@ -391,12 +389,6 @@ class Thd_ndb {
   */
   void clear_ddl_transaction_ctx();
 
-  /*
-    @brief  Create a string to identify the owner thread of this Thd_ndb
-    @return String identification of the owner thread
-  */
-  std::string get_info_str() const;
-
  private:
   std::unique_ptr<Ndb_applier> m_applier;
 
@@ -429,27 +421,4 @@ class Thd_ndb {
   Ndb_applier *get_applier() const { return m_applier.get(); }
 };
 
-/**
-  @brief RAII style class for seizing and relasing a Thd_ndb
-*/
-class Thd_ndb_guard {
-  THD *const m_thd;
-  Thd_ndb *const m_thd_ndb;
-  Thd_ndb_guard() = delete;
-  Thd_ndb_guard(const Thd_ndb_guard &) = delete;
-
- public:
-  Thd_ndb_guard(THD *thd, const char *name)
-      : m_thd(thd), m_thd_ndb(Thd_ndb::seize(m_thd, name)) {
-    thd_set_thd_ndb(m_thd, m_thd_ndb);
-  }
-
-  ~Thd_ndb_guard() {
-    Thd_ndb::release(m_thd_ndb);
-    thd_set_thd_ndb(m_thd, nullptr);
-  }
-
-  const Thd_ndb *get_thd_ndb() const { return m_thd_ndb; }
-};
-
 #endif
