@@ -105,8 +105,8 @@ DECLARE_NDBINFO_TABLE(POOLS, 14) = {
      {"resource_id", Ndbinfo::Number, ""},
      {"type_id", Ndbinfo::Number, "Record type id within resource"}}};
 
-DECLARE_NDBINFO_TABLE(TRANSPORTER_DETAILS, 19) = {
-    {"transporter_details", 19, 0,
+DECLARE_NDBINFO_TABLE(TRANSPORTER_DETAILS, 18) = {
+    {"transporter_details", 18, 0,
      [](const Ndbinfo::Counts &counts) {
        return (counts.data_nodes) * (counts.all_nodes - 1);
      },
@@ -139,12 +139,10 @@ DECLARE_NDBINFO_TABLE(TRANSPORTER_DETAILS, 19) = {
      {"sendbuffer_alloc_bytes", Ndbinfo::Number64,
       "SendBuffer bytes allocated"},
      {"sendbuffer_max_alloc_bytes", Ndbinfo::Number64,
-      "SendBuffer historical max bytes allocated"},
+      "SendBuffer historical max bytes allocated"}}};
 
-     {"type", Ndbinfo::Number, "Transporter type"}}};
-
-DECLARE_NDBINFO_TABLE(TRANSPORTERS, 11) = {
-    {"transporters", 11, 0,
+DECLARE_NDBINFO_TABLE(TRANSPORTERS, 12) = {
+    {"transporters", 12, 0,
      [](const Ndbinfo::Counts &counts) {
        return (counts.data_nodes) * (counts.all_nodes - 1);
      },
@@ -166,7 +164,8 @@ DECLARE_NDBINFO_TABLE(TRANSPORTERS, 11) = {
 
      {"slowdown", Ndbinfo::Number, "Is link requesting slowdown"},
      {"slowdown_count", Ndbinfo::Number,
-      "Number of slowdown onsets since connect"}}};
+      "Number of slowdown onsets since connect"},
+     {"encrypted", Ndbinfo::Number, "Is link using TLS encryption"}}};
 
 DECLARE_NDBINFO_TABLE(LOGSPACES, 7) = {
     {"logspaces", 7, 0,
@@ -647,7 +646,7 @@ DECLARE_NDBINFO_TABLE(TC_TIME_TRACK_STATS, 15) = {
 DECLARE_NDBINFO_TABLE(CONFIG_VALUES, 3) = {
     {"config_values", 3, 0,
      [](const Ndbinfo::Counts &c) {
-       return c.data_nodes * 165;  // 165 = current number of config parameters
+       return c.data_nodes * 167;  // 167 = current number of config parameters
      },
      "Configuration parameter values"},
     {
@@ -1202,6 +1201,16 @@ DECLARE_NDBINFO_TABLE(CPUDATA_20SEC, 10) = {
          "Elapsed time in microseconds for measurement"},
     }};
 
+DECLARE_NDBINFO_TABLE(CERTIFICATES, 5) = {
+    {"certificates", 5, 0,
+     [](const Ndbinfo::Counts &c) { return c.data_nodes * c.all_nodes; },
+     "Certificates in current use for TLS connections"},
+    {{"reporting_node_id", Ndbinfo::Number, "Reporting node"},
+     {"node_id", Ndbinfo::Number, "Peer node"},
+     {"name", Ndbinfo::String, "Certificate subject common name"},
+     {"serial", Ndbinfo::String, "Certificate serial number"},
+     {"expires", Ndbinfo::Number, "Certificate expiration date"}}};
+
 #define DBINFOTBL(x) \
   { Ndbinfo::x##_TABLEID, (const Ndbinfo::Table *)&ndbinfo_##x }
 
@@ -1263,7 +1272,7 @@ static struct ndbinfo_table_list_entry {
     DBINFOTBL(CPUDATA_50MS),
     DBINFOTBL(CPUDATA_1SEC),
     DBINFOTBL(CPUDATA_20SEC),
-    DBINFOTBL_UNSUPPORTED(CERTIFICATES),
+    DBINFOTBL(CERTIFICATES),
     DBINFOTBL(THREADBLOCK_DETAILS),
     DBINFOTBL(TRANSPORTER_DETAILS)};
 
