@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1000,6 +1001,7 @@ int runBug14019036(NDBT_Context *ctx, NDBT_Step *step) {
 
   res.insertErrorInAllNodes(5073);  // slow down backup
 
+  mgmd.use_tls(opt_tls_search_path, opt_mgm_tls);
   if (!mgmd.connect()) {
     g_err << "Cannot connect to mgmd server" << endl;
     return NDBT_FAILED;
@@ -1175,6 +1177,7 @@ int runBug17882305(NDBT_Context *ctx, NDBT_Step *step) {
     return NDBT_FAILED;
   }
 
+  res.insertErrorInAllNodes(0);
   return NDBT_OK;
 }
 
@@ -1905,6 +1908,7 @@ int runCheckPrintout(NDBT_Context *ctx, NDBT_Step *step) {
   NdbRestarter res;
   NdbMgmd mgmd;
 
+  mgmd.use_tls(opt_tls_search_path, opt_mgm_tls);
   CHECK2(mgmd.connect());
   CHECK2(mgmd.subscribe_to_events());
 
