@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -55,7 +56,7 @@ class PFS_index_ets_by_account_by_event_name : public PFS_engine_index {
         m_key_2("HOST"),
         m_key_3("EVENT_NAME") {}
 
-  ~PFS_index_ets_by_account_by_event_name() override {}
+  ~PFS_index_ets_by_account_by_event_name() override = default;
 
   virtual bool match(PFS_account *pfs);
   virtual bool match(PFS_instr_class *instr_class);
@@ -92,12 +93,12 @@ struct row_ets_by_account_by_event_name {
 struct pos_ets_by_account_by_event_name : public PFS_double_index {
   pos_ets_by_account_by_event_name() : PFS_double_index(0, 1) {}
 
-  inline void reset(void) {
+  inline void reset() {
     m_index_1 = 0;
     m_index_2 = 1;
   }
 
-  inline void next_account(void) {
+  inline void next_account() {
     m_index_1++;
     m_index_2 = 1;
   }
@@ -113,7 +114,7 @@ class table_ets_by_account_by_event_name : public PFS_engine_table {
   static int delete_all_rows();
   static ha_rows get_row_count();
 
-  void reset_position(void) override;
+  void reset_position() override;
 
   int rnd_init(bool scan) override;
   int rnd_next() override;
@@ -129,7 +130,7 @@ class table_ets_by_account_by_event_name : public PFS_engine_table {
   table_ets_by_account_by_event_name();
 
  public:
-  ~table_ets_by_account_by_event_name() override {}
+  ~table_ets_by_account_by_event_name() override = default;
 
  protected:
   int make_row(PFS_account *account, PFS_transaction_class *klass);
