@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +27,11 @@
 #include "storage/perfschema/pfs_global.h"
 #include "storage/perfschema/pfs_instr.h"
 #include "storage/perfschema/pfs_instr_class.h"
+#include "storage/perfschema/unittest/stub_digest.h"
 #include "storage/perfschema/unittest/stub_pfs_plugin_table.h"
+#include "storage/perfschema/unittest/stub_pfs_tls_channel.h"
+#include "storage/perfschema/unittest/stub_server_telemetry.h"
+#include "storage/perfschema/unittest/stub_telemetry_metrics.h"
 #include "unittest/mytap/tap.h"
 
 static void test_no_registration() {
@@ -43,7 +48,11 @@ static void test_no_registration() {
   PFS_file_class *file;
   PFS_socket_class *socket;
   PFS_memory_class *memory;
+  PFS_meter_class *meter;
+  PFS_metric_class *metric;
   /* PFS_table_share *table; */
+  PFS_meter_key meter_key;
+  PFS_metric_key metric_key;
 
   rc = init_sync_class(0, 0, 0);
   ok(rc == 0, "zero init (sync)");
@@ -57,6 +66,10 @@ static void test_no_registration() {
   ok(rc == 0, "zero init (table)");
   rc = init_memory_class(0);
   ok(rc == 0, "zero init (memory)");
+  rc = init_meter_class(0);
+  ok(rc == 0, "zero init (meter)");
+  rc = init_metric_class(0);
+  ok(rc == 0, "zero init (metric)");
 
   PSI_mutex_info_v1 mutex_info;
   memset(&mutex_info, 0, sizeof(mutex_info));
@@ -67,8 +80,9 @@ static void test_no_registration() {
   PSI_cond_info_v1 cond_info;
   memset(&cond_info, 0, sizeof(cond_info));
 
-  PSI_thread_info_v1 thread_info;
+  PSI_thread_info_v5 thread_info;
   memset(&thread_info, 0, sizeof(thread_info));
+  thread_info.m_os_name = "OS_NAME";
 
   PSI_file_info_v1 file_info;
   memset(&file_info, 0, sizeof(file_info));
@@ -79,6 +93,12 @@ static void test_no_registration() {
   PSI_memory_info_v1 memory_info;
   memset(&memory_info, 0, sizeof(memory_info));
 
+  PSI_meter_info_v1 meter_info;
+  memset(&meter_info, 0, sizeof(meter_info));
+
+  PSI_metric_info_v1 metric_info;
+  memset(&metric_info, 0, sizeof(metric_info));
+
   key = register_mutex_class("FOO", 3, &mutex_info);
   ok(key == 0, "no mutex registered");
   key = register_mutex_class("BAR", 3, &mutex_info);
@@ -128,6 +148,20 @@ static void test_no_registration() {
   memory_key = register_memory_class("FOO", 3, &memory_info);
   ok(memory_key == 0, "no memory registered");
 
+  meter_key = register_meter_class("FOO", 3, &meter_info);
+  ok(meter_key == 0, "no meter registered");
+  meter_key = register_meter_class("BAR", 3, &meter_info);
+  ok(meter_key == 0, "no meter registered");
+  meter_key = register_meter_class("FOO", 3, &meter_info);
+  ok(meter_key == 0, "no meter registered");
+
+  metric_key = register_metric_class("FOO", 3, &metric_info, "meter");
+  ok(metric_key == 0, "no metric registered");
+  metric_key = register_metric_class("BAR", 3, &metric_info, "meter");
+  ok(metric_key == 0, "no metric registered");
+  metric_key = register_metric_class("FOO", 3, &metric_info, "meter");
+  ok(metric_key == 0, "no metric registered");
+
 #ifdef LATER
   PFS_thread fake_thread;
   fake_thread.m_table_share_hash_pins = NULL;
@@ -192,12 +226,28 @@ static void test_no_registration() {
   memory = find_memory_class(9999);
   ok(memory == nullptr, "no memory key 9999");
 
+  meter = find_meter_class(0);
+  ok(meter == nullptr, "no meter key 0");
+  meter = find_meter_class(1);
+  ok(meter == nullptr, "no meter key 1");
+  meter = find_meter_class(9999);
+  ok(meter == nullptr, "no meter key 9999");
+
+  metric = find_metric_class(0);
+  ok(metric == nullptr, "no metric key 0");
+  metric = find_metric_class(1);
+  ok(metric == nullptr, "no metric key 1");
+  metric = find_metric_class(9999);
+  ok(metric == nullptr, "no metric key 9999");
+
   cleanup_sync_class();
   cleanup_thread_class();
   cleanup_file_class();
   cleanup_socket_class();
   cleanup_table_share();
   cleanup_memory_class();
+  cleanup_meter_class();
+  cleanup_metric_class();
 }
 
 static void test_mutex_registration() {
@@ -240,8 +290,8 @@ static void test_mutex_registration() {
   ok(mutex == nullptr, "no key 0");
   mutex = find_mutex_class(3);
   ok(mutex != nullptr, "found key 3");
-  ok(strncmp(mutex->m_name, "M-3", 3) == 0, "key 3 is M-3");
-  ok(mutex->m_name_length == 3, "name length 3");
+  ok(strncmp(mutex->m_name.str(), "M-3", 3) == 0, "key 3 is M-3");
+  ok(mutex->m_name.length() == 3, "name length 3");
   mutex = find_mutex_class(9999);
   ok(mutex == nullptr, "no key 9999");
 
@@ -283,8 +333,8 @@ static void test_rwlock_registration() {
   ok(rwlock == nullptr, "no key 0");
   rwlock = find_rwlock_class(3);
   ok(rwlock != nullptr, "found key 3");
-  ok(strncmp(rwlock->m_name, "RW-3", 4) == 0, "key 3 is RW-3");
-  ok(rwlock->m_name_length == 4, "name length 4");
+  ok(strncmp(rwlock->m_name.str(), "RW-3", 4) == 0, "key 3 is RW-3");
+  ok(rwlock->m_name.length() == 4, "name length 4");
   rwlock = find_rwlock_class(9999);
   ok(rwlock == nullptr, "no key 9999");
 
@@ -326,8 +376,8 @@ static void test_cond_registration() {
   ok(cond == nullptr, "no key 0");
   cond = find_cond_class(3);
   ok(cond != nullptr, "found key 3");
-  ok(strncmp(cond->m_name, "C-3", 3) == 0, "key 3 is C-3");
-  ok(cond->m_name_length == 3, "name length 3");
+  ok(strncmp(cond->m_name.str(), "C-3", 3) == 0, "key 3 is C-3");
+  ok(cond->m_name.length() == 3, "name length 3");
   cond = find_cond_class(9999);
   ok(cond == nullptr, "no key 9999");
 
@@ -338,8 +388,9 @@ static void test_thread_registration() {
   int rc;
   PFS_thread_key key;
   PFS_thread_class *thread;
-  PSI_thread_info_v1 thread_info;
+  PSI_thread_info_v5 thread_info;
   memset(&thread_info, 0, sizeof(thread_info));
+  thread_info.m_os_name = "OS_NAME";
 
   rc = init_thread_class(5);
   ok(rc == 0, "room for 5 thread");
@@ -369,8 +420,8 @@ static void test_thread_registration() {
   ok(thread == nullptr, "no key 0");
   thread = find_thread_class(3);
   ok(thread != nullptr, "found key 3");
-  ok(strncmp(thread->m_name, "Thread-3", 8) == 0, "key 3 is Thread-3");
-  ok(thread->m_name_length == 8, "name length 8");
+  ok(strncmp(thread->m_name.str(), "Thread-3", 8) == 0, "key 3 is Thread-3");
+  ok(thread->m_name.length() == 8, "name length 8");
   thread = find_thread_class(9999);
   ok(thread == nullptr, "no key 9999");
 
@@ -412,8 +463,8 @@ static void test_file_registration() {
   ok(file == nullptr, "no key 0");
   file = find_file_class(3);
   ok(file != nullptr, "found key 3");
-  ok(strncmp(file->m_name, "File-3", 6) == 0, "key 3 is File-3");
-  ok(file->m_name_length == 6, "name length 6");
+  ok(strncmp(file->m_name.str(), "File-3", 6) == 0, "key 3 is File-3");
+  ok(file->m_name.length() == 6, "name length 6");
   file = find_file_class(9999);
   ok(file == nullptr, "no key 9999");
 
@@ -460,8 +511,8 @@ static void test_socket_registration() {
   ok(socket == nullptr, "no key 0");
   socket = find_socket_class(3);
   ok(socket != nullptr, "found key 3");
-  ok(strncmp(socket->m_name, "Socket-3", 8) == 0, "key 3 is Socket-3");
-  ok(socket->m_name_length == 8, "name length 3");
+  ok(strncmp(socket->m_name.str(), "Socket-3", 8) == 0, "key 3 is Socket-3");
+  ok(socket->m_name.length() == 8, "name length 3");
   socket = find_socket_class(9999);
   ok(socket == nullptr, "no key 9999");
 
@@ -577,14 +628,109 @@ static void test_memory_registration() {
   ok(memory == nullptr, "no key 0");
   memory = find_memory_class(3);
   ok(memory != nullptr, "found key 3");
-  ok(strncmp(memory->m_name, "Memory-3", 8) == 0, "key 3 is Memory-3");
-  ok(memory->m_name_length == 8, "name length 3");
+  ok(strncmp(memory->m_name.str(), "Memory-3", 8) == 0, "key 3 is Memory-3");
+  ok(memory->m_name.length() == 8, "name length 3");
   memory = find_memory_class(9999);
   ok(memory == nullptr, "no key 9999");
 
   cleanup_memory_class();
 }
 
+static void test_meter_registration() {
+  int rc;
+  PFS_meter_key key;
+  PFS_meter_class *meter;
+  PSI_meter_info_v1 meter_info;
+  memset(&meter_info, 0, sizeof(meter_info));
+
+  // meter stores metrics
+  init_metric_class(5);
+  rc = init_meter_class(5);
+  ok(rc == 0, "room for 5 meter");
+
+  key = register_meter_class("FOO", 3, &meter_info);
+  ok(key == 1, "foo registered");
+  key = register_meter_class("BAR", 3, &meter_info);
+  ok(key == 2, "bar registered");
+  key = register_meter_class("FOO", 3, &meter_info);
+  ok(key == UINT_MAX, "foo duplicate detected");
+  key = register_meter_class("meter-3", 7, &meter_info);
+  ok(key == 3, "meter-3 registered");
+  key = register_meter_class("meter-4", 7, &meter_info);
+  ok(key == 4, "meter-4 registered");
+  key = register_meter_class("meter-5", 7, &meter_info);
+  ok(key == 5, "meter-5 registered");
+  ok(meter_class_lost == 0, "lost nothing");
+  key = register_meter_class("meter-6", 7, &meter_info);
+  ok(key == 0, "meter-6 not registered");
+  ok(meter_class_lost == 1, "lost 1 meter");
+  key = register_meter_class("meter-7", 7, &meter_info);
+  ok(key == 0, "meter-7 not registered");
+  ok(meter_class_lost == 2, "lost 2 meter");
+  key = register_meter_class("meter-3", 7, &meter_info);
+  ok(key == UINT_MAX, "meter-3 duplicate detected");
+  ok(meter_class_lost == 2, "lost 2 meter");
+  key = register_meter_class("meter-5", 7, &meter_info);
+  ok(key == UINT_MAX, "meter-5 duplicate detected");
+  ok(meter_class_lost == 2, "lost 2 meter");
+
+  meter = find_meter_class(0);
+  ok(meter == nullptr, "no key 0");
+  meter = find_meter_class(3);
+  ok(meter != nullptr, "found key 3");
+  meter = find_meter_class(9999);
+  ok(meter == nullptr, "no key 9999");
+
+  cleanup_meter_class();
+  cleanup_metric_class();
+}
+
+static void test_metric_registration() {
+  int rc;
+  PFS_metric_key key;
+  PFS_metric_class *metric;
+  PSI_metric_info_v1 metric_info;
+  memset(&metric_info, 0, sizeof(metric_info));
+
+  rc = init_metric_class(5);
+  ok(rc == 0, "room for 5 metric");
+
+  key = register_metric_class("FOO", 3, &metric_info, "meter");
+  ok(key == 1, "foo registered");
+  key = register_metric_class("BAR", 3, &metric_info, "meter");
+  ok(key == 2, "bar registered");
+  key = register_metric_class("FOO", 3, &metric_info, "meter");
+  ok(key == UINT_MAX, "foo duplicate detected");
+  key = register_metric_class("metric-3", 8, &metric_info, "meter");
+  ok(key == 3, "metric-3 registered");
+  key = register_metric_class("metric-4", 8, &metric_info, "meter");
+  ok(key == 4, "metric-4 registered");
+  key = register_metric_class("metric-5", 8, &metric_info, "meter");
+  ok(key == 5, "metric-5 registered");
+  ok(metric_class_lost == 0, "lost nothing");
+  key = register_metric_class("metric-6", 8, &metric_info, "meter");
+  ok(key == 0, "metric-6 not registered");
+  ok(metric_class_lost == 1, "lost 1 metric");
+  key = register_metric_class("metric-7", 8, &metric_info, "meter");
+  ok(key == 0, "metric-7 not registered");
+  ok(metric_class_lost == 2, "lost 2 metric");
+  key = register_metric_class("metric-3", 8, &metric_info, "meter");
+  ok(key == UINT_MAX, "metric-3 duplicate detected");
+  ok(metric_class_lost == 2, "lost 2 metric");
+  key = register_metric_class("metric-5", 8, &metric_info, "meter");
+  ok(key == UINT_MAX, "metric-5 duplicate detected");
+  ok(metric_class_lost == 2, "lost 2 metric");
+
+  metric = find_metric_class(0);
+  ok(metric == nullptr, "no key 0");
+  metric = find_metric_class(3);
+  ok(metric != nullptr, "found key 3");
+  metric = find_metric_class(9999);
+  ok(metric == nullptr, "no key 9999");
+
+  cleanup_metric_class();
+}
+
 #ifdef LATER
 void set_wait_stat(PFS_instr_class *klass) {
   PFS_single_stat *stat;
@@ -781,11 +927,13 @@ static void do_all_tests() {
   test_socket_registration();
   test_table_registration();
   test_memory_registration();
+  test_meter_registration();
+  test_metric_registration();
   test_instruments_reset();
 }
 
 int main(int, char **) {
-  plan(209);
+  plan(261);
   MY_INIT("pfs_instr_info-t");
   do_all_tests();
   my_end(0);
