@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -128,6 +129,11 @@ class mock_gcs_xcom_communication_interface
                void(Gcs_protocol_version version));
   MOCK_CONST_METHOD0(get_maximum_supported_protocol_version,
                      Gcs_protocol_version());
+  MOCK_METHOD1(set_communication_protocol,
+               void(enum_transport_protocol protocol));
+  MOCK_METHOD1(add_communication_provider,
+               void(std::shared_ptr<Network_provider> provider));
+  MOCK_METHOD0(get_incoming_connections_protocol, enum_transport_protocol());
 
   virtual Gcs_message_pipeline &get_msg_pipeline() { return m_msg_pipeline; }
 
@@ -137,7 +143,7 @@ class mock_gcs_xcom_communication_interface
 
 class XComStateExchangeTest : public GcsBaseTest {
  protected:
-  XComStateExchangeTest() {}
+  XComStateExchangeTest() = default;
 
   void SetUp() override {
     control_mock = new mock_gcs_control_interface();
@@ -267,7 +273,8 @@ TEST_F(XComStateExchangeTest, StateExchangeBroadcastJoinerTest) {
 
 uchar *copied_payload = nullptr;
 uint64_t copied_length = 0;
-enum_gcs_error copy_message_content(const Gcs_message &msg) {
+enum_gcs_error copy_message_content(const Gcs_message &msg,
+                                    unsigned long long *, Cargo_type) {
   copied_length = msg.get_message_data().get_payload_length();
   copied_payload = static_cast<uchar *>(malloc(sizeof(uchar) * copied_length));
   memcpy(copied_payload, msg.get_message_data().get_payload(), copied_length);
@@ -277,7 +284,7 @@ enum_gcs_error copy_message_content(const Gcs_message &msg) {
 
 TEST_F(XComStateExchangeTest, StateExchangeProcessStatesPhase) {
   EXPECT_CALL(*comm_mock, do_send_message(_, _, _))
-      .WillOnce(WithArgs<0>(Invoke(copy_message_content)));
+      .WillOnce(Invoke(copy_message_content));
 
   /*
     Define that the first view delivered has two members, i.e.
